@@ -1,7 +1,7 @@
 ############################################################################
 #
-# Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-# Contact: http://www.qt-project.org/legal
+# Copyright (C) 2015 The Qt Company Ltd.
+# Contact: http://www.qt.io/licensing
 #
 # This file is part of Qt Creator.
 #
@@ -9,8 +9,8 @@
 # Licensees holding valid commercial Qt licenses may use this file in
 # accordance with the commercial license agreement provided with the
 # Software or, alternatively, in accordance with the terms contained in
-# a written agreement between you and Digia.  For licensing terms and
-# conditions see http://www.qt.io/licensing.  For further information
+# a written agreement between you and The Qt Company.  For licensing terms and
+# conditions see http://www.qt.io/terms-conditions.  For further information
 # use the contact form at http://www.qt.io/contact-us.
 #
 # GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 # requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 # http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 #
-# In addition, as a special exception, Digia gives you certain additional
-# rights.  These rights are described in the Digia Qt LGPL Exception
+# In addition, as a special exception, The Qt Company gives you certain additional
+# rights.  These rights are described in The Qt Company LGPL Exception
 # version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 #
 #############################################################################
@@ -53,21 +53,26 @@ def qdump__QAtomicPointer(d, value):
            d.putSubItem("_q_value", q.dereference())
 
 def qform__QByteArray():
-    return "Inline,As Latin1 in Separate Window,As UTF-8 in Separate Window"
+    return [Latin1StringFormat, SeparateLatin1StringFormat,
+            Utf8StringFormat, SeparateUtf8StringFormat ]
 
 def qdump__QByteArray(d, value):
-    d.putByteArrayValue(value)
     data, size, alloc = d.byteArrayData(value)
     d.putNumChild(size)
-    format = d.currentItemFormat()
-    if format == 1:
-        d.putDisplay(StopDisplay)
-    elif format == 2:
+    elided, p = d.encodeByteArrayHelper(d.extractPointer(value), d.displayStringLimit)
+    displayFormat = d.currentItemFormat()
+    if displayFormat == AutomaticFormat or displayFormat == Latin1StringFormat:
+        d.putValue(p, Hex2EncodedLatin1, elided=elided)
+    elif displayFormat == SeparateLatin1StringFormat:
+        d.putValue(p, Hex2EncodedLatin1, elided=elided)
         d.putField("editformat", DisplayLatin1String)
-        d.putField("editvalue", d.encodeByteArray(value))
-    elif format == 3:
+        d.putField("editvalue", d.encodeByteArray(value, limit=100000))
+    elif displayFormat == Utf8StringFormat:
+        d.putValue(p, Hex2EncodedUtf8, elided=elided)
+    elif displayFormat == SeparateUtf8StringFormat:
+        d.putValue(p, Hex2EncodedUtf8, elided=elided)
         d.putField("editformat", DisplayUtf8String)
-        d.putField("editvalue", d.encodeByteArray(value))
+        d.putField("editvalue", d.encodeByteArray(value, limit=100000))
     if d.isExpanded():
         d.putArrayData(data, size, d.charType())
 
@@ -86,14 +91,14 @@ def qdump__QChar(d, value):
 
 
 def qform__QAbstractItemModel():
-    return "Normal,Enhanced"
+    return [SimpleFormat, EnhancedFormat]
 
 def qdump__QAbstractItemModel(d, value):
-    format = d.currentItemFormat()
-    if format == 1:
+    displayFormat = d.currentItemFormat()
+    if displayFormat == SimpleFormat:
         d.putPlainChildren(value)
         return
-    #format == 2:
+    #displayFormat == EnhancedFormat:
     # Create a default-constructed QModelIndex on the stack.
     try:
         ri = d.makeValue(d.qtNamespace() + "QModelIndex", "-1, -1, 0, 0")
@@ -128,11 +133,11 @@ def qdump__QAbstractItemModel(d, value):
     #gdb.execute("call free($ri)")
 
 def qform__QModelIndex():
-    return "Normal,Enhanced"
+    return [SimpleFormat, EnhancedFormat]
 
 def qdump__QModelIndex(d, value):
-    format = d.currentItemFormat()
-    if format == 1:
+    displayFormat = d.currentItemFormat()
+    if displayFormat == SimpleFormat:
         d.putPlainChildren(value)
         return
     r = value["r"]
@@ -168,9 +173,10 @@ def qdump__QModelIndex(d, value):
     except:
         d.putValue("")
 
-    d.putNumChild(rowCount * columnCount)
+    d.putNumChild(1)
     if d.isExpanded():
         with Children(d):
+            d.putFields(value, False)
             i = 0
             for row in xrange(rowCount):
                 for column in xrange(columnCount):
@@ -180,12 +186,7 @@ def qdump__QModelIndex(d, value):
                             % (mm_, row, column, mi_))
                         d.putItem(mi2)
                         i = i + 1
-            d.putFields(value)
-            #d.putCallItem("parent", val, "parent")
-            #with SubItem(d, "model"):
-            #    d.putValue(m)
-            #    d.putType(ns + "QAbstractItemModel*")
-            #    d.putNumChild(1)
+            d.putCallItem("parent", value, "parent")
     #gdb.execute("call free($mi)")
 
 
@@ -197,14 +198,15 @@ def qdump__QDate(d, value):
         if d.isExpanded():
             # FIXME: This improperly uses complex return values.
             with Children(d):
-                d.putCallItem("toString", value, "toString",
-                    d.enumExpression("DateFormat", "TextDate"))
-                d.putCallItem("(ISO)", value, "toString",
-                    d.enumExpression("DateFormat", "ISODate"))
-                d.putCallItem("(SystemLocale)", value, "toString",
-                    d.enumExpression("DateFormat", "SystemLocaleDate"))
-                d.putCallItem("(Locale)", value, "toString",
-                    d.enumExpression("DateFormat", "LocaleDate"))
+                if d.canCallLocale():
+                    d.putCallItem("toString", value, "toString",
+                        d.enumExpression("DateFormat", "TextDate"))
+                    d.putCallItem("(ISO)", value, "toString",
+                        d.enumExpression("DateFormat", "ISODate"))
+                    d.putCallItem("(SystemLocale)", value, "toString",
+                        d.enumExpression("DateFormat", "SystemLocaleDate"))
+                    d.putCallItem("(Locale)", value, "toString",
+                        d.enumExpression("DateFormat", "LocaleDate"))
                 d.putFields(value)
     else:
         d.putValue("(invalid)")
@@ -223,10 +225,11 @@ def qdump__QTime(d, value):
                     d.enumExpression("DateFormat", "TextDate"))
                 d.putCallItem("(ISO)", value, "toString",
                     d.enumExpression("DateFormat", "ISODate"))
-                d.putCallItem("(SystemLocale)", value, "toString",
-                    d.enumExpression("DateFormat", "SystemLocaleDate"))
-                d.putCallItem("(Locale)", value, "toString",
-                    d.enumExpression("DateFormat", "LocaleDate"))
+                if d.canCallLocale():
+                    d.putCallItem("(SystemLocale)", value, "toString",
+                        d.enumExpression("DateFormat", "SystemLocaleDate"))
+                    d.putCallItem("(Locale)", value, "toString",
+                        d.enumExpression("DateFormat", "LocaleDate"))
                 d.putFields(value)
     else:
         d.putValue("(invalid)")
@@ -304,18 +307,19 @@ def qdump__QDateTime(d, value):
             # FIXME: This improperly uses complex return values.
             with Children(d):
                 d.putCallItem("toTime_t", value, "toTime_t")
-                d.putCallItem("toString", value, "toString",
-                    d.enumExpression("DateFormat", "TextDate"))
-                d.putCallItem("(ISO)", value, "toString",
-                    d.enumExpression("DateFormat", "ISODate"))
-                d.putCallItem("(SystemLocale)", value, "toString",
-                    d.enumExpression("DateFormat", "SystemLocaleDate"))
-                d.putCallItem("(Locale)", value, "toString",
-                    d.enumExpression("DateFormat", "LocaleDate"))
-                d.putCallItem("toUTC", value, "toTimeSpec",
-                    d.enumExpression("TimeSpec", "UTC"))
-                d.putCallItem("toLocalTime", value, "toTimeSpec",
-                    d.enumExpression("TimeSpec", "LocalTime"))
+                if d.canCallLocale():
+                    d.putCallItem("toString", value, "toString",
+                        d.enumExpression("DateFormat", "TextDate"))
+                    d.putCallItem("(ISO)", value, "toString",
+                        d.enumExpression("DateFormat", "ISODate"))
+                    d.putCallItem("toUTC", value, "toTimeSpec",
+                        d.enumExpression("TimeSpec", "UTC"))
+                    d.putCallItem("(SystemLocale)", value, "toString",
+                        d.enumExpression("DateFormat", "SystemLocaleDate"))
+                    d.putCallItem("(Locale)", value, "toString",
+                        d.enumExpression("DateFormat", "LocaleDate"))
+                    d.putCallItem("toLocalTime", value, "toTimeSpec",
+                        d.enumExpression("TimeSpec", "LocalTime"))
                 d.putFields(value)
     else:
         d.putValue("(invalid)")
@@ -413,10 +417,20 @@ def qdump__QDir(d, value):
 
 
 def qdump__QFile(d, value):
-    # 9fc0965 changes the layout of the private structure
+    # 9fc0965 and a373ffcd change the layout of the private structure
     qtVersion = d.qtVersion()
     is32bit = d.is32bit()
-    if qtVersion > 0x050200:
+    if qtVersion >= 0x050500:
+        if d.isWindowsTarget():
+            offset = 164 if is32bit else 248
+        else:
+            offset = 156 if is32bit else 248
+    elif qtVersion >= 0x050400:
+        if d.isWindowsTarget():
+            offset = 188 if is32bit else 272
+        else:
+            offset = 180 if is32bit else 272
+    elif qtVersion > 0x050200:
         if d.isWindowsTarget():
             offset = 180 if is32bit else 272
         else:
@@ -533,30 +547,8 @@ def qdump__QFiniteStack(d, value):
     size = int(value["_size"])
     d.check(0 <= size and size <= alloc and alloc <= 1000 * 1000 * 1000)
     d.putItemCount(size)
-    if d.isExpanded():
-        d.putPlotData(value["_array"], size, d.templateArgument(value.type, 0))
+    d.putPlotData(value["_array"], size, d.templateArgument(value.type, 0))
 
-# Stock gdb 7.2 seems to have a problem with types here:
-#
-#  echo -e "namespace N { struct S { enum E { zero, one, two }; }; }\n"\
-#      "int main() { N::S::E x = N::S::one;\n return x; }" >> main.cpp
-#  g++ -g main.cpp
-#  gdb-7.2 -ex 'file a.out' -ex 'b main' -ex 'run' -ex 'step' \
-#     -ex 'ptype N::S::E' -ex 'python print gdb.lookup_type("N::S::E")' -ex 'q'
-#  gdb-7.1 -ex 'file a.out' -ex 'b main' -ex 'run' -ex 'step' \
-#     -ex 'ptype N::S::E' -ex 'python print gdb.lookup_type("N::S::E")' -ex 'q'
-#  gdb-cvs -ex 'file a.out' -ex 'b main' -ex 'run' -ex 'step' \
-#     -ex 'ptype N::S::E' -ex 'python print gdb.lookup_type("N::S::E")' -ex 'q'
-#
-# gives as of 2010-11-02
-#
-#  type = enum N::S::E {N::S::zero, N::S::one, N::S::two} \n
-#    Traceback (most recent call last): File "<string>", line 1,
-#      in <module> RuntimeError: No type named N::S::E.
-#  type = enum N::S::E {N::S::zero, N::S::one, N::S::two} \n  N::S::E
-#  type = enum N::S::E {N::S::zero, N::S::one, N::S::two} \n  N::S::E
-#
-# i.e. there's something broken in stock 7.2 that is was ok in 7.1 and is ok later.
 
 def qdump__QFlags(d, value):
     i = value["i"]
@@ -761,7 +753,7 @@ def qdump__QIPv6Address(d, value):
     d.putPlainChildren(c)
 
 def qform__QList():
-    return "Assume Direct Storage,Assume Indirect Storage"
+    return [DirectQListStorageFormat, IndirectQListStorageFormat]
 
 def qdump__QList(d, value):
     base = d.extractPointer(value)
@@ -787,10 +779,10 @@ def qdump__QList(d, value):
         # but this data is available neither in the compiled binary nor
         # in the frontend.
         # So as first approximation only do the 'isLarge' check:
-        format = d.currentItemFormat()
-        if format == 1:
+        displayFormat = d.currentItemFormat()
+        if displayFormat == DirectQListStorageFormat:
             isInternal = True
-        elif format == 2:
+        elif displayFormat == IndirectQListStorageFormat:
             isInternal = False
         else:
             isInternal = innerSize <= stepSize and d.isMovableType(innerType)
@@ -811,7 +803,7 @@ def qdump__QList(d, value):
                     d.putSubItem(i, x)
 
 def qform__QImage():
-    return "Normal,Displayed"
+    return [SimpleFormat, SeparateFormat]
 
 def qdump__QImage(d, value):
     # This relies on current QImage layout:
@@ -854,10 +846,8 @@ def qdump__QImage(d, value):
                 d.putNumChild(0)
                 d.putType("void *")
 
-    format = d.currentItemFormat()
-    if format == 1:
-        d.putDisplay(StopDisplay)
-    elif format == 2:
+    displayFormat = d.currentItemFormat()
+    if displayFormat == SeparateFormat:
         # This is critical for performance. Writing to an external
         # file using the following is faster when using GDB.
         #   file = tempfile.mkstemp(prefix="gdbpy_")
@@ -1049,16 +1039,6 @@ def qdump__QMultiMap(d, value):
     qdump__QMap(d, value)
 
 
-def extractCString(table, offset):
-    result = ""
-    while True:
-        d = table[offset]
-        if d == 0:
-            break
-        result += "%c" % d
-        offset += 1
-    return result
-
 def qdump__QMetaObjectPrivate(d, value):
     d.putEmptyValue()
     d.putNumChild(1)
@@ -1591,14 +1571,22 @@ def qdump__QRegion(d, value):
                 ns = d.qtNamespace()
                 rectType = d.lookupType(ns + "QRect")
                 d.putIntItem("numRects", n)
-                d.putSubItem("extents", d.createValue(pp + 2 * v, rectType))
-                d.putSubItem("innerRect", d.createValue(pp + 2 * v + rectType.sizeof, rectType))
-                d.putIntItem("innerArea", d.extractInt(pp + 2 * v + 2 * rectType.sizeof))
+                if d.qtVersion() >= 0x050400:
+                    # Changed in ee324e4ed
+                    d.putSubItem("extents", d.createValue(pp + 8 + v, rectType))
+                    d.putSubItem("innerRect", d.createValue(pp + 8 + v + rectType.sizeof, rectType))
+                    d.putIntItem("innerArea", d.extractInt(pp + 4))
+                    rectsOffset = 8
+                else:
+                    d.putSubItem("extents", d.createValue(pp + 2 * v, rectType))
+                    d.putSubItem("innerRect", d.createValue(pp + 2 * v + rectType.sizeof, rectType))
+                    d.putIntItem("innerArea", d.extractInt(pp + 2 * v + 2 * rectType.sizeof))
+                    rectsOffset = v
                 # FIXME
                 try:
                     # Can fail if QVector<QRect> debuginfo is missing.
                     vectType = d.lookupType("%sQVector<%sQRect>" % (ns, ns))
-                    d.putSubItem("rects", d.createValue(pp + v, vectType))
+                    d.putSubItem("rects", d.createValue(pp + rectsOffset, vectType))
                 except:
                     with SubItem(d, "rects"):
                         d.putItemCount(n)
@@ -1663,10 +1651,6 @@ def qdump__QSet(d, value):
                 it = node.dereference().cast(innerType)
                 with SubItem(d, i):
                     key = it["key"]
-                    if not key:
-                        # LLDB can't access directly since it's in anonymous union
-                        # for Qt4 optimized int keytype
-                        key = it[1]["key"]
                     d.putItem(key)
 
 
@@ -1716,6 +1700,21 @@ def qform__QStack():
 def qdump__QStack(d, value):
     qdump__QVector(d, value)
 
+def qdump__QPolygonF(d, value):
+    qdump__QVector(d, value.cast(d.directBaseClass(value.type, 0)))
+    d.putBetterType(d.currentType)
+
+def qdump__QPolygon(d, value):
+    qdump__QVector(d, value.cast(d.directBaseClass(value.type, 0)))
+    d.putBetterType(d.currentType)
+
+def qdump__QGraphicsPolygonItem(d, value):
+    dptr = d.extractPointer(d.addressOf(value) + d.ptrSize()) # Skip vtable
+    # Assume sizeof(QGraphicsPolygonItemPrivate) == 400
+    offset = 308 if d.is32bit() else 384
+    data, size, alloc = d.vectorDataHelper(d.extractPointer(dptr + offset))
+    d.putItemCount(size)
+    d.putPlotData(data, size, d.lookupQtType("QPointF"))
 
 def qdump__QStandardItem(d, value):
     d.putBetterType(d.currentType)
@@ -1731,18 +1730,16 @@ def qedit__QString(d, value, data):
     d.setValues(base, "short", [ord(c) for c in data])
 
 def qform__QString():
-    return "Inline,Separate Window"
+    return [SimpleFormat, SeparateFormat]
 
 def qdump__QString(d, value):
     d.putStringValue(value)
     data, size, alloc = d.stringData(value)
     d.putNumChild(size)
-    format = d.currentItemFormat()
-    if format == 1:
-        d.putDisplay(StopDisplay)
-    elif format == 2:
+    displayFormat = d.currentItemFormat()
+    if displayFormat == SeparateFormat:
         d.putField("editformat", DisplayUtf16String)
-        d.putField("editvalue", d.encodeString(value, limit=None))
+        d.putField("editvalue", d.encodeString(value, limit=100000))
     if d.isExpanded():
         d.putArrayData(data, size, d.lookupType(d.qtNamespace() + "QChar"))
 
@@ -1827,7 +1824,7 @@ def qdump__QTextDocument(d, value):
 
 
 def qform__QUrl():
-    return "Inline,Separate Window"
+    return [SimpleFormat, SeparateFormat]
 
 def qdump__QUrl(d, value):
     if d.qtVersion() < 0x050000:
@@ -1891,10 +1888,8 @@ def qdump__QUrl(d, value):
         url += path
         d.putValue(url, Hex4EncodedLittleEndian)
 
-        format = d.currentItemFormat()
-        if format == 1:
-            d.putDisplay(StopDisplay)
-        elif format == 2:
+        displayFormat = d.currentItemFormat()
+        if displayFormat == SeparateFormat:
             d.putField("editformat", DisplayUtf16String)
             d.putField("editvalue", url)
 
@@ -1910,6 +1905,7 @@ def qdump__QUrl(d, value):
                 d.putGenericItem("path", stringType, path, Hex4EncodedLittleEndian)
                 d.putGenericItem("query", stringType, query, Hex4EncodedLittleEndian)
                 d.putGenericItem("fragment", stringType, fragment, Hex4EncodedLittleEndian)
+                d.putFields(value)
 
 def qdumpHelper_QVariant_0(d, blob):
     # QVariant::Invalid
@@ -2188,6 +2184,12 @@ def qdump__QVector(d, value):
     d.putItemCount(size)
     d.putPlotData(data, size, d.templateArgument(value.type, 0))
 
+def qdump__QVarLengthArray(d, value):
+    data = d.extractPointer(value["ptr"])
+    size = int(value["s"])
+    d.check(0 <= size)
+    d.putItemCount(size)
+    d.putPlotData(data, size, d.templateArgument(value.type, 0))
 
 def qdump__QWeakPointer(d, value):
     d_ptr = value["d"]
@@ -2250,28 +2252,94 @@ def qdump__QXmlStreamAttribute(d, value):
 #
 #######################################################################
 
-def qdump__QV4__String(d, value):
-    d.putStringValue(value["identifier"]["string"])
-    d.putNumChild(0)
+def qdump__QV4__Object(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__FunctionObject(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__CompilationUnit(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__CallContext(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__ScriptFunction(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__SimpleScriptFunction(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
+
+def qdump__QV4__ExecutionContext(d, value):
+    d.putBetterType(d.currentType)
+    d.putItem(d.extractQmlData(value))
 
 def qdump__QV4__TypedValue(d, value):
+    d.putBetterType(d.currentType)
     qdump__QV4__Value(d, d.directBaseObject(value))
-    d.putBetterType(value.type)
+
+def qdump__QV4__CallData(d, value):
+    argc = toInteger(value["argc"])
+    d.putItemCount(argc)
+    if d.isExpanded():
+        with Children(d):
+            d.putSubItem("[this]", value["thisObject"])
+            for i in range(0, argc):
+                d.putSubItem(i, value["args"][i])
+
+def qdump__QV4__String(d, value):
+    d.putStringValue(d.addressOf(value) + 2 * d.ptrSize())
 
 def qdump__QV4__Value(d, value):
-    try:
-        if d.is64bit():
-            vtable = value["m"]["internalClass"]["vtable"]
-            if toInteger(vtable["isString"]):
-                d.putBetterType(d.qtNamespace() + "QV4::Value (String)")
-                d.putStringValue(value["s"]["identifier"]["string"])
+    v = toInteger(str(value["val"]))
+    NaNEncodeMask          = 0xffff800000000000
+    IsInt32Mask            = 0x0002000000000000
+    IsDoubleMask           = 0xfffc000000000000
+    IsNumberMask           = IsInt32Mask | IsDoubleMask
+    IsNullOrUndefinedMask  = 0x0000800000000000
+    IsNullOrBooleanMask    = 0x0001000000000000
+    IsConvertibleToIntMask = IsInt32Mask | IsNullOrBooleanMask
+    ns = d.qtNamespace()
+    if v & IsInt32Mask:
+        d.putBetterType("%sQV4::Value (int32)" % ns)
+        d.putValue(value["int_32"])
+    elif v & IsDoubleMask:
+        d.putBetterType("%sQV4::Value (double)" % ns)
+        d.putValue("%x" % (v ^ 0xffff800000000000), Hex2EncodedFloat8)
+    elif d.isNull(v):
+        d.putBetterType("%sQV4::Value (null)" % ns)
+        d.putValue("(null)")
+    elif v & IsNullOrUndefinedMask:
+        d.putBetterType("%sQV4::Value (null/undef)" % ns)
+        d.putValue("(null/undef)")
+    elif v & IsNullOrBooleanMask:
+        d.putBetterType("%sQV4::Value (null/bool)" % ns)
+        d.putValue("(null/bool)")
+    else:
+        vtable = value["m"]["vtable"]
+        if toInteger(vtable["isString"]):
+            d.putBetterType("%sQV4::Value (string)" % ns)
+            d.putStringValue(d.extractPointer(value) + 2 * d.ptrSize())
+        elif toInteger(vtable["isObject"]):
+            d.putBetterType("%sQV4::Value (object)" % ns)
+            d.putValue("[0x%x]" % v)
+        else:
+            d.putBetterType("%sQV4::Value (unknown)" % ns)
+            d.putValue("[0x%x]" % v)
+    d.putNumChild(1)
+    if d.isExpanded():
+        with Children(d):
+            with SubItem(d, "[raw]"):
+                d.putValue("[0x%x]" % v)
+                d.putType(" ");
                 d.putNumChild(0)
-                return
-    except:
-        pass
-
-    # Fall back for cases that we do not handle specifically.
-    d.putPlainChildren(value)
+            d.putFields(value)
 
 
 #######################################################################
