@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -249,13 +249,16 @@ SynchronousProcess::SynchronousProcess() :
     d(new SynchronousProcessPrivate)
 {
     d->m_timer.setInterval(1000);
-    connect(&d->m_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
-    connect(&d->m_process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(finished(int,QProcess::ExitStatus)));
-    connect(&d->m_process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(error(QProcess::ProcessError)));
-    connect(&d->m_process, SIGNAL(readyReadStandardOutput()),
-            this, SLOT(stdOutReady()));
-    connect(&d->m_process, SIGNAL(readyReadStandardError()),
-            this, SLOT(stdErrReady()));
+    connect(&d->m_timer, &QTimer::timeout, this, &SynchronousProcess::slotTimeout);
+    connect(&d->m_process,
+            static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
+            this, &SynchronousProcess::finished);
+    connect(&d->m_process, static_cast<void (QProcess::*)(QProcess::ProcessError)>(&QProcess::error),
+            this, &SynchronousProcess::error);
+    connect(&d->m_process, &QProcess::readyReadStandardOutput,
+            this, &SynchronousProcess::stdOutReady);
+    connect(&d->m_process, &QProcess::readyReadStandardError,
+            this, &SynchronousProcess::stdErrReady);
 }
 
 SynchronousProcess::~SynchronousProcess()
@@ -385,7 +388,7 @@ static bool isGuiThread()
 }
 
 SynchronousProcessResponse SynchronousProcess::run(const QString &binary,
-                                                 const QStringList &args)
+                                                   const QStringList &args)
 {
     if (debug)
         qDebug() << '>' << Q_FUNC_INFO << binary << args;
@@ -615,14 +618,7 @@ bool SynchronousProcess::readDataFromProcess(QProcess &p, int timeOutMS,
         }
         // Prompt user, pretend we have data if says 'No'.
         const bool hang = !hasData && !finished;
-        if (hang && showTimeOutMessageBox) {
-            QString binary;
-#if QT_VERSION >= 0x050000
-            binary = p.program();
-#endif
-            if (!askToKill(binary))
-                hasData = true;
-        }
+        hasData = hang && showTimeOutMessageBox && !askToKill(p.program());
     } while (hasData && !finished);
     if (syncDebug)
         qDebug() << "<readDataFromProcess" << finished;
