@@ -45,8 +45,6 @@
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
 
-#include <utils/pathchooser.h>
-
 #include <QFile>
 #include <QTextStream>
 #include <QProcess>
@@ -170,7 +168,6 @@ AndroidSettingsWidget::AndroidSettingsWidget(QWidget *parent)
     m_ui->DataPartitionSizeSpinBox->setValue(m_androidConfig.partitionSize());
     m_ui->CreateKitCheckBox->setChecked(m_androidConfig.automaticKitCreation());
     m_ui->AVDTableView->setModel(&m_AVDModel);
-    m_AVDModel.setAvdList(m_androidConfig.androidVirtualDevices());
     m_ui->AVDTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
     m_ui->AVDTableView->horizontalHeader()->setSectionResizeMode(1, QHeaderView::ResizeToContents);
 
@@ -180,11 +177,20 @@ AndroidSettingsWidget::AndroidSettingsWidget(QWidget *parent)
     connect(m_ui->gdbWarningLabel, SIGNAL(linkActivated(QString)),
             this, SLOT(showGdbWarningDialog()));
 
+    connect(&m_virtualDevicesWatcher, SIGNAL(finished()),
+            this, SLOT(updateAvds()));
+
     check(All);
     applyToUi(All);
 
     connect(&m_futureWatcher, SIGNAL(finished()),
             this, SLOT(avdAdded()));
+
+    connect(m_ui->NDKLocationPathChooser, SIGNAL(changed(QString)), this, SLOT(ndkLocationEditingFinished()));
+    connect(m_ui->SDKLocationPathChooser, SIGNAL(changed(QString)), this, SLOT(sdkLocationEditingFinished()));
+    connect(m_ui->AntLocationPathChooser, SIGNAL(changed(QString)), this, SLOT(antLocationEditingFinished()));
+    connect(m_ui->OpenJDKLocationPathChooser, SIGNAL(changed(QString)), this, SLOT(openJDKLocationEditingFinished()));
+
 }
 
 AndroidSettingsWidget::~AndroidSettingsWidget()
@@ -274,9 +280,9 @@ void AndroidSettingsWidget::check(AndroidSettingsWidget::Mode mode)
             QStringList gdbPaths;
             foreach (const AndroidToolChainFactory::AndroidToolChainInformation &ati, compilerPaths) {
                 // we only check the arm gdbs, that's indicative enough
-                if (ati.architecture != ProjectExplorer::Abi::ArmArchitecture)
+                if (ati.abi.architecture() != ProjectExplorer::Abi::ArmArchitecture)
                     continue;
-                Utils::FileName gdbPath = m_androidConfig.gdbPath(ati.architecture, ati.version);
+                Utils::FileName gdbPath = m_androidConfig.gdbPath(ati.abi, ati.version);
                 if (gdbPath.exists())
                     gdbPaths << gdbPath.toString();
             }
@@ -287,32 +293,29 @@ void AndroidSettingsWidget::check(AndroidSettingsWidget::Mode mode)
             }
 
             // See if we have qt versions for those toolchains
-            QSet<ProjectExplorer::Abi::Architecture> toolchainsForArch;
+            QSet<ProjectExplorer::Abi> toolchainsForAbi;
             foreach (const AndroidToolChainFactory::AndroidToolChainInformation &ati, compilerPaths)
-                toolchainsForArch.insert(ati.architecture);
+                toolchainsForAbi.insert(ati.abi);
 
-            QSet<ProjectExplorer::Abi::Architecture> qtVersionsForArch;
+            QSet<ProjectExplorer::Abi> qtVersionsForAbi;
             foreach (QtSupport::BaseQtVersion *qtVersion, QtSupport::QtVersionManager::versions()) {
                 if (qtVersion->type() != QLatin1String(Constants::ANDROIDQT) || qtVersion->qtAbis().isEmpty())
                     continue;
-                qtVersionsForArch.insert(qtVersion->qtAbis().first().architecture());
+                qtVersionsForAbi.insert(qtVersion->qtAbis().first());
             }
 
-            QSet<ProjectExplorer::Abi::Architecture> missingQtArchs = toolchainsForArch.subtract(qtVersionsForArch);
+            QSet<ProjectExplorer::Abi> missingQtArchs = toolchainsForAbi.subtract(qtVersionsForAbi);
             if (missingQtArchs.isEmpty()) {
                 m_ndkMissingQtArchs.clear();
             } else {
                 if (missingQtArchs.count() == 1) {
                     m_ndkMissingQtArchs = tr("Qt version for architecture %1 is missing.\n"
                                              "To add the Qt version, select Options > Build & Run > Qt Versions.")
-                            .arg(ProjectExplorer::Abi::toString((*missingQtArchs.constBegin())));
+                            .arg((*missingQtArchs.constBegin()).toString());
                 } else {
-                    QStringList missingArchs;
-                    foreach (ProjectExplorer::Abi::Architecture arch, missingQtArchs)
-                        missingArchs.append(ProjectExplorer::Abi::toString(arch));
-                    m_ndkMissingQtArchs = tr("Qt versions for architectures %1 are missing.\n"
+                    m_ndkMissingQtArchs = tr("Qt versions for %1 architectures are missing.\n"
                                              "To add the Qt versions, select Options > Build & Run > Qt Versions.")
-                            .arg(missingArchs.join(QLatin1String(", ")));
+                            .arg(missingQtArchs.size());
                 }
             }
         }
@@ -397,10 +400,41 @@ void AndroidSettingsWidget::applyToUi(AndroidSettingsWidget::Mode mode)
             m_ui->AVDManagerFrame->setEnabled(false);
         }
 
-        m_AVDModel.setAvdList(m_androidConfig.androidVirtualDevices());
+        startUpdateAvd();
     }
 }
 
+void AndroidSettingsWidget::disableAvdControls()
+{
+    m_ui->AVDAddPushButton->setEnabled(false);
+    m_ui->AVDTableView->setEnabled(false);
+    m_ui->AVDRemovePushButton->setEnabled(false);
+    m_ui->AVDStartPushButton->setEnabled(false);
+}
+
+void AndroidSettingsWidget::enableAvdControls()
+{
+    m_ui->AVDTableView->setEnabled(true);
+    m_ui->AVDAddPushButton->setEnabled(true);
+    avdActivated(m_ui->AVDTableView->currentIndex());
+}
+
+void AndroidSettingsWidget::startUpdateAvd()
+{
+    disableAvdControls();
+    m_virtualDevicesWatcher.setFuture(m_androidConfig.androidVirtualDevicesFuture());
+}
+
+void AndroidSettingsWidget::updateAvds()
+{
+    m_AVDModel.setAvdList(m_virtualDevicesWatcher.result());
+    if (!m_lastAddedAvd.isEmpty()) {
+        m_ui->AVDTableView->setCurrentIndex(m_AVDModel.indexForAvdName(m_lastAddedAvd));
+        m_lastAddedAvd.clear();
+    }
+    enableAvdControls();
+}
+
 bool AndroidSettingsWidget::sdkLocationIsValid() const
 {
     Utils::FileName androidExe = m_androidConfig.sdkLocation();
@@ -519,11 +553,11 @@ void AndroidSettingsWidget::openOpenJDKDownloadUrl()
 
 void AndroidSettingsWidget::addAVD()
 {
-    m_ui->AVDAddPushButton->setEnabled(false);
+    disableAvdControls();
     AndroidConfig::CreateAvdInfo info = m_androidConfig.gatherCreateAVDInfo(this);
 
     if (info.target.isEmpty()) {
-        m_ui->AVDAddPushButton->setEnabled(true);
+        enableAvdControls();
         return;
     }
 
@@ -532,29 +566,31 @@ void AndroidSettingsWidget::addAVD()
 
 void AndroidSettingsWidget::avdAdded()
 {
-    m_ui->AVDAddPushButton->setEnabled(true);
     AndroidConfig::CreateAvdInfo info = m_futureWatcher.result();
     if (!info.error.isEmpty()) {
+        enableAvdControls();
         QMessageBox::critical(this, QApplication::translate("AndroidConfig", "Error Creating AVD"), info.error);
         return;
     }
 
-    m_AVDModel.setAvdList(m_androidConfig.androidVirtualDevices());
-    m_ui->AVDTableView->setCurrentIndex(m_AVDModel.indexForAvdName(info.name));
+    startUpdateAvd();
+    m_lastAddedAvd = info.name;
 }
 
 void AndroidSettingsWidget::removeAVD()
 {
+    disableAvdControls();
     QString avdName = m_AVDModel.avdName(m_ui->AVDTableView->currentIndex());
     if (QMessageBox::question(this, tr("Remove Android Virtual Device"),
                               tr("Remove device \"%1\"? This cannot be undone.").arg(avdName),
                               QMessageBox::Yes | QMessageBox::No)
-            == QMessageBox::No)
+            == QMessageBox::No) {
+        enableAvdControls();
         return;
+    }
 
     m_androidConfig.removeAVD(avdName);
-    m_AVDModel.setAvdList(m_androidConfig.androidVirtualDevices());
-    avdActivated(m_ui->AVDTableView->currentIndex());
+    startUpdateAvd();
 }
 
 void AndroidSettingsWidget::startAVD()
@@ -562,7 +598,7 @@ void AndroidSettingsWidget::startAVD()
     m_androidConfig.startAVDAsync(m_AVDModel.avdName(m_ui->AVDTableView->currentIndex()));
 }
 
-void AndroidSettingsWidget::avdActivated(QModelIndex index)
+void AndroidSettingsWidget::avdActivated(const QModelIndex &index)
 {
     m_ui->AVDRemovePushButton->setEnabled(index.isValid());
     m_ui->AVDStartPushButton->setEnabled(index.isValid());
@@ -598,8 +634,8 @@ void AndroidSettingsWidget::showGdbWarningDialog()
                          tr("Unsupported GDB"),
                          tr("The GDB inside this NDK seems to not support Python. "
                             "The Qt Project offers fixed GDB builds at: "
-                            "<a href=\"http://download.qt-project.org/official_releases/gdb/\">"
-                            "http://download.qt-project.org/official_releases/gdb/</a>"));
+                            "<a href=\"http://download.qt.io/official_releases/gdb/\">"
+                            "http://download.qt.io/official_releases/gdb/</a>"));
 }
 
 void AndroidSettingsWidget::manageAVD()
