@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -145,6 +145,7 @@ private:
     QToolButton *m_statusWidgetButton;
     QList<IEditor *> m_editors;
     QMap<QWidget *, IEditor *> m_widgetEditorMap;
+    QLabel *m_emptyViewLabel;
 
     QList<EditLocation> m_navigationHistory;
     QList<EditLocation> m_editorHistory;
@@ -173,7 +174,7 @@ class SplitterOrView  : public QWidget
 {
     Q_OBJECT
 public:
-    explicit SplitterOrView(Core::IEditor *editor = 0);
+    explicit SplitterOrView(IEditor *editor = 0);
     explicit SplitterOrView(EditorView *view);
     ~SplitterOrView();
 
@@ -183,9 +184,9 @@ public:
     inline bool isView() const { return m_view != 0; }
     inline bool isSplitter() const { return m_splitter != 0; }
 
-    inline Core::IEditor *editor() const { return m_view ? m_view->currentEditor() : 0; }
-    inline QList<Core::IEditor *> editors() const { return m_view ? m_view->editors() : QList<Core::IEditor*>(); }
-    inline bool hasEditor(Core::IEditor *editor) const { return m_view && m_view->hasEditor(editor); }
+    inline IEditor *editor() const { return m_view ? m_view->currentEditor() : 0; }
+    inline QList<IEditor *> editors() const { return m_view ? m_view->editors() : QList<IEditor*>(); }
+    inline bool hasEditor(IEditor *editor) const { return m_view && m_view->hasEditor(editor); }
     inline bool hasEditors() const { return m_view && m_view->editorCount() != 0; }
     inline EditorView *view() const { return m_view; }
     inline QSplitter *splitter() const { return m_splitter; }
