@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -37,8 +37,9 @@
 #include <debugger/debuggerstringutils.h>
 #include <qmldebug/qmldebugclient.h>
 
+#include <coreplugin/messagebox.h>
+
 #include <QFileInfo>
-#include <QMessageBox>
 #include <utils/qtcassert.h>
 
 using QmlDebug::QmlDebugStream;
@@ -219,9 +220,9 @@ void QScriptDebuggerClient::startSession()
     BreakHandler *handler = d->engine->breakHandler();
     DebuggerEngine * engine = d->engine->isSlaveEngine() ?
                 d->engine->masterEngine() : d->engine;
-    foreach (BreakpointModelId id, handler->engineBreakpointIds(engine)) {
-        QTC_CHECK(handler->state(id) == BreakpointInsertProceeding);
-        handler->notifyBreakpointInsertOk(id);
+    foreach (Breakpoint bp, handler->engineBreakpoints(engine)) {
+        QTC_CHECK(bp.state() == BreakpointInsertProceeding);
+        bp.notifyBreakpointInsertOk();
     }
     d->sessionStarted = true;
 }
@@ -246,46 +247,42 @@ void QScriptDebuggerClient::activateFrame(int index)
     sendMessage(reply);
 }
 
-void QScriptDebuggerClient::insertBreakpoint(const BreakpointModelId &id,
+void QScriptDebuggerClient::insertBreakpoint(Breakpoint bp,
                                              int adjustedLine,
                                              int /*adjustedColumn*/)
 {
-    BreakHandler *handler = d->engine->breakHandler();
-    JSAgentBreakpointData bp;
-    bp.fileUrl = QUrl::fromLocalFile(handler->fileName(id)).toString().toUtf8();
-    bp.lineNumber = adjustedLine;
-    bp.functionName = handler->functionName(id).toUtf8();
-    d->breakpoints.insert(bp);
+    JSAgentBreakpointData jsbp;
+    jsbp.fileUrl = QUrl::fromLocalFile(bp.fileName()).toString().toUtf8();
+    jsbp.lineNumber = adjustedLine;
+    jsbp.functionName = bp.functionName().toUtf8();
+    d->breakpoints.insert(jsbp);
 
-    BreakpointResponse br = handler->response(id);
+    BreakpointResponse br = bp.response();
     br.lineNumber = adjustedLine;
-    handler->setResponse(id, br);
-    if (d->sessionStarted && handler->state(id) == BreakpointInsertProceeding)
-        handler->notifyBreakpointInsertOk(id);
+    bp.setResponse(br);
+    if (d->sessionStarted && bp.state() == BreakpointInsertProceeding)
+        bp.notifyBreakpointInsertOk();
 }
 
-void QScriptDebuggerClient::removeBreakpoint(const BreakpointModelId &id)
+void QScriptDebuggerClient::removeBreakpoint(Breakpoint bp)
 {
-    BreakHandler *handler = d->engine->breakHandler();
-    JSAgentBreakpointData bp;
-    bp.fileUrl = QUrl::fromLocalFile(handler->fileName(id)).toString().toUtf8();
-    bp.lineNumber = handler->lineNumber(id);
-    bp.functionName = handler->functionName(id).toUtf8();
-    d->breakpoints.remove(bp);
+    JSAgentBreakpointData jsbp;
+    jsbp.fileUrl = QUrl::fromLocalFile(bp.fileName()).toString().toUtf8();
+    jsbp.lineNumber = bp.lineNumber();
+    jsbp.functionName = bp.functionName().toUtf8();
+    d->breakpoints.remove(jsbp);
 }
 
-void QScriptDebuggerClient::changeBreakpoint(const BreakpointModelId &id)
+void QScriptDebuggerClient::changeBreakpoint(Breakpoint bp)
 {
-    BreakHandler *handler = d->engine->breakHandler();
-    if (handler->isEnabled(id)) {
-        BreakpointResponse br = handler->response(id);
-        insertBreakpoint(id, br.lineNumber);
-    } else {
-        removeBreakpoint(id);
-    }
-    BreakpointResponse br = handler->response(id);
-    br.enabled = handler->isEnabled(id);
-    handler->setResponse(id, br);
+    if (bp.isEnabled())
+        insertBreakpoint(bp, bp.response().lineNumber);
+    else
+        removeBreakpoint(bp);
+
+    BreakpointResponse br = bp.response();
+    br.enabled = bp.isEnabled();
+    bp.setResponse(br);
 }
 
 void QScriptDebuggerClient::synchronizeBreakpoints()
@@ -445,10 +442,10 @@ void QScriptDebuggerClient::messageReceived(const QByteArray &data)
 
             QString msg = stackFrames.isEmpty()
                     ? tr("<p>An uncaught exception occurred:</p><p>%1</p>")
-                      .arg(Qt::escape(error))
+                      .arg(error.toHtmlEscaped())
                     : tr("<p>An uncaught exception occurred in \"%1\":</p><p>%2</p>")
-                      .arg(QLatin1String(stackFrames.value(0).fileUrl), Qt::escape(error));
-            showMessageBox(QMessageBox::Information, tr("Uncaught Exception"), msg);
+                      .arg(QLatin1String(stackFrames.value(0).fileUrl), error.toHtmlEscaped());
+            Core::AsynchronousMessageBox::information(tr("Uncaught Exception"), msg);
         } else {
             QString file;
             int line = -1;
@@ -485,16 +482,18 @@ void QScriptDebuggerClient::messageReceived(const QByteArray &data)
 
         d->logReceiveMessage(QLatin1String(command) + QLatin1Char(' ')
                              +  QLatin1String(iname) + QLatin1Char(' ') + data.value);
-        data.iname = iname;
+
+        auto item = new WatchItem(data);
+        item->iname = iname;
         if (iname.startsWith("watch.")) {
-            watchHandler->insertData(data);
+            watchHandler->insertItem(item);
         } else if (iname == "console") {
-            d->engine->showMessage(data.value, ConsoleOutput);
+            d->engine->showMessage(item->value, ConsoleOutput);
         } else if (iname.startsWith("local.")) {
-            data.name = data.name.left(data.name.indexOf(QLatin1Char(' ')));
-            watchHandler->insertData(data);
+            item->name = item->name.left(item->name.indexOf(QLatin1Char(' ')));
+            watchHandler->insertItem(item);
         } else {
-            qWarning() << "QmlEngine: Unexcpected result: " << iname << data.value;
+            qWarning() << "QmlEngine: Unexcpected result: " << iname << item->value;
         }
     } else if (command == "EXPANDED") {
         QList<WatchData> result;
@@ -507,7 +506,7 @@ void QScriptDebuggerClient::messageReceived(const QByteArray &data)
 
         foreach (WatchData data, result) {
             data.iname = iname + '.' + data.exp;
-            watchHandler->insertData(data);
+            watchHandler->insertItem(new WatchItem(data));
 
             if (watchHandler->isExpandedIName(data.iname) && qint64(data.id) != -1) {
                 needPing = true;
@@ -555,9 +554,10 @@ void QScriptDebuggerClient::insertLocalsAndWatches(QList<WatchData> &locals,
         return;
 
     bool needPing = false;
-    foreach (WatchData data, watches) {
-        data.iname = watchHandler->watcherName(data.exp);
-        watchHandler->insertData(data);
+    foreach (const WatchData &data, watches) {
+        auto item = new WatchItem(data);
+        item->iname = watchHandler->watcherName(data.exp);
+        watchHandler->insertItem(item);
 
         if (watchHandler->isExpandedIName(data.iname) && qint64(data.id) != -1) {
             needPing = true;
@@ -565,11 +565,12 @@ void QScriptDebuggerClient::insertLocalsAndWatches(QList<WatchData> &locals,
         }
     }
 
-    foreach (WatchData data, locals) {
-        if (data.name == QLatin1String("<no initialized data>"))
-            data.name = tr("No Local Variables");
-        data.iname = "local." + data.exp;
-        watchHandler->insertData(data);
+    foreach (const WatchData &data, locals) {
+        auto item = new WatchItem(data);
+        if (item->name == QLatin1String("<no initialized data>"))
+            item->name = tr("No Local Variables");
+        item->iname = "local." + item->exp;
+        watchHandler->insertItem(item);
 
         if (watchHandler->isExpandedIName(data.iname) && qint64(data.id) != -1) {
             needPing = true;
@@ -585,7 +586,8 @@ void QScriptDebuggerClient::insertLocalsAndWatches(QList<WatchData> &locals,
 void QScriptDebuggerClient::setEngine(QmlEngine *engine)
 {
     d->engine = engine;
-    connect(this, SIGNAL(stackFrameCompleted()), engine, SIGNAL(stackFrameCompleted()));
+    connect(this, &QScriptDebuggerClient::stackFrameCompleted,
+            engine, &DebuggerEngine::stackFrameCompleted);
 }
 
 void QScriptDebuggerClientPrivate::logSendMessage(const QString &msg) const
