@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -35,15 +35,11 @@
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/nodesvisitor.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/session.h>
 #include <resourceeditor/resourcenode.h>
 
-#if QT_VERSION >= 0x050000
-#    include <QDesignerFormWindowInterface>
-#else
-#    include "qt_private/formwindowbase_p.h"
-#    include "qt_private/qtresourcemodel_p.h"
-#endif
+#include <QDesignerFormWindowInterface>
 
 #include <utils/qtcassert.h>
 
@@ -77,56 +73,42 @@ void QrcFilesVisitor::visitProjectNode(ProjectNode *projectNode)
 void QrcFilesVisitor::visitFolderNode(FolderNode *folderNode)
 {
     foreach (const FileNode *fileNode, folderNode->fileNodes()) {
-        if (fileNode->fileType() == ProjectExplorer::ResourceType)
-            m_qrcFiles.append(fileNode->path());
+        if (fileNode->fileType() == ResourceType)
+            m_qrcFiles.append(fileNode->path().toString());
     }
-    if (qobject_cast<ResourceEditor::ResourceTopLevelNode *>(folderNode))
-        m_qrcFiles.append(folderNode->path());
+    if (dynamic_cast<ResourceEditor::ResourceTopLevelNode *>(folderNode))
+        m_qrcFiles.append(folderNode->path().toString());
 }
 
 // ------------ ResourceHandler
-#if QT_VERSION >= 0x050000
 ResourceHandler::ResourceHandler(QDesignerFormWindowInterface *fw) :
-#else
-ResourceHandler::ResourceHandler(qdesigner_internal::FormWindowBase *fw) :
-#endif
     QObject(fw),
     m_form(fw),
-    m_sessionNode(0),
-    m_sessionWatcher(0),
+    m_initialized(false),
     m_handlingResources(false)
 {
 }
 
 void ResourceHandler::ensureInitialized()
 {
-    if (m_sessionNode)
+    if (m_initialized)
         return;
 
-    m_sessionNode = ProjectExplorer::SessionManager::sessionNode();
-    m_sessionWatcher = new ProjectExplorer::NodesWatcher();
+    m_initialized = true;
+    ProjectTree *tree = ProjectTree::instance();
 
-    connect(m_sessionWatcher, SIGNAL(filesAdded()), this, SLOT(updateResources()));
-    connect(m_sessionWatcher, SIGNAL(filesRemoved()), this, SLOT(updateResources()));
-    connect(m_sessionWatcher, SIGNAL(foldersAdded()), this, SLOT(updateResources()));
-    connect(m_sessionWatcher, SIGNAL(foldersRemoved()), this, SLOT(updateResources()));
-    m_sessionNode->registerWatcher(m_sessionWatcher);
-#if QT_VERSION >= 0x050000
+    connect(tree, SIGNAL(filesAdded()), this, SLOT(updateResources()));
+    connect(tree, SIGNAL(filesRemoved()), this, SLOT(updateResources()));
+    connect(tree, SIGNAL(foldersAdded()), this, SLOT(updateResources()));
+    connect(tree, SIGNAL(foldersRemoved()), this, SLOT(updateResources()));
     m_originalUiQrcPaths = m_form->activeResourceFilePaths();
-#else
-    m_originalUiQrcPaths = m_form->resourceSet()->activeQrcPaths();
-#endif
     if (Designer::Constants::Internal::debug)
         qDebug() << "ResourceHandler::ensureInitialized() origPaths=" << m_originalUiQrcPaths;
 }
 
 ResourceHandler::~ResourceHandler()
 {
-    // Close: Delete the Designer form window via embedding widget
-    if (m_sessionNode && m_sessionWatcher) {
-        m_sessionNode->unregisterWatcher(m_sessionWatcher);
-        delete m_sessionWatcher;
-    }
+
 }
 
 void ResourceHandler::updateResources(bool updateProjectResources)
@@ -143,7 +125,8 @@ void ResourceHandler::updateResources(bool updateProjectResources)
         qDebug() << "ResourceHandler::updateResources()" << fileName;
 
     // Filename could change in the meantime.
-    Project *project = SessionManager::projectForFile(fileName);
+    Project *project = SessionManager::projectForFile(
+                Utils::FileName::fromUserInput(QDir::fromNativeSeparators(fileName)));
     const bool dirty = m_form->property("_q_resourcepathchanged").toBool();
     if (dirty)
         m_form->setDirty(true);
@@ -170,26 +153,16 @@ void ResourceHandler::updateResources(bool updateProjectResources)
             }
         }
 
-#if QT_VERSION >= 0x050000
         m_form->activateResourceFilePaths(projectQrcFiles);
         m_form->setResourceFileSaveMode(QDesignerFormWindowInterface::SaveOnlyUsedResourceFiles);
-#else
-        m_form->resourceSet()->activateQrcPaths(projectQrcFiles);
-        m_form->setSaveResourcesBehaviour(qdesigner_internal::FormWindowBase::SaveOnlyUsedQrcFiles);
-#endif
         if (Designer::Constants::Internal::debug)
             qDebug() << "ResourceHandler::updateResources()" << fileName
                     << " associated with project" << project->rootProjectNode()->path()
                     <<  " using project qrc files" << projectQrcFiles.size();
     } else {
         // Use resource file originally used in form
-#if QT_VERSION >= 0x050000
         m_form->activateResourceFilePaths(m_originalUiQrcPaths);
         m_form->setResourceFileSaveMode(QDesignerFormWindowInterface::SaveAllResourceFiles);
-#else
-        m_form->resourceSet()->activateQrcPaths(m_originalUiQrcPaths);
-        m_form->setSaveResourcesBehaviour(qdesigner_internal::FormWindowBase::SaveAll);
-#endif
         if (Designer::Constants::Internal::debug)
             qDebug() << "ResourceHandler::updateResources()" << fileName << " not associated with project, using loaded qrc files.";
     }
