@@ -31,12 +31,13 @@
 #include "diffeditorplugin.h"
 #include "diffeditor.h"
 #include "diffeditorconstants.h"
+#include "diffeditorcontroller.h"
 #include "diffeditordocument.h"
 #include "diffeditorfactory.h"
 #include "diffeditormanager.h"
-#include "diffeditorreloader.h"
 #include "differ.h"
 
+#include <QAction>
 #include <QFileDialog>
 #include <QTextCodec>
 #include <QtPlugin>
@@ -47,15 +48,17 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 
+#include <utils/qtcassert.h>
+
 namespace DiffEditor {
 namespace Internal {
 
-class SimpleDiffEditorReloader : public DiffEditorReloader
+class FileDiffController : public DiffEditorController
 {
     Q_OBJECT
 public:
-    SimpleDiffEditorReloader(const QString &leftFileName,
-                             const QString &rightFileName);
+    FileDiffController(Core::IDocument *document, const QString &leftFileName,
+                       const QString &rightFileName);
 
 protected:
     void reload();
@@ -65,14 +68,12 @@ private:
     QString m_rightFileName;
 };
 
-SimpleDiffEditorReloader::SimpleDiffEditorReloader(const QString &leftFileName,
-                                                   const QString &rightFileName)
-    : m_leftFileName(leftFileName),
-      m_rightFileName(rightFileName)
-{
-}
+FileDiffController::FileDiffController(Core::IDocument *document, const QString &leftFileName,
+                                       const QString &rightFileName) :
+    DiffEditorController(document), m_leftFileName(leftFileName), m_rightFileName(rightFileName)
+{ }
 
-void SimpleDiffEditorReloader::reload()
+void FileDiffController::reload()
 {
     QString errorString;
     Utils::TextFileFormat format;
@@ -83,7 +84,6 @@ void SimpleDiffEditorReloader::reload()
                                     format.codec,
                                     &leftText, &format, &errorString)
             != Utils::TextFileFormat::ReadSuccess) {
-
         return;
     }
 
@@ -92,13 +92,11 @@ void SimpleDiffEditorReloader::reload()
                                     format.codec,
                                     &rightText, &format, &errorString)
             != Utils::TextFileFormat::ReadSuccess) {
-
         return;
     }
 
     Differ differ;
-    QList<Diff> diffList = differ.cleanupSemantics(
-                differ.diff(leftText, rightText));
+    QList<Diff> diffList = differ.cleanupSemantics(differ.diff(leftText, rightText));
 
     QList<Diff> leftDiffList;
     QList<Diff> rightDiffList;
@@ -106,15 +104,13 @@ void SimpleDiffEditorReloader::reload()
     QList<Diff> outputLeftDiffList;
     QList<Diff> outputRightDiffList;
 
-    if (controller()->isIgnoreWhitespace()) {
-        const QList<Diff> leftIntermediate =
-                Differ::moveWhitespaceIntoEqualities(leftDiffList);
-        const QList<Diff> rightIntermediate =
-                Differ::moveWhitespaceIntoEqualities(rightDiffList);
-        Differ::ignoreWhitespaceBetweenEqualities(leftIntermediate,
-                                                  rightIntermediate,
-                                                  &outputLeftDiffList,
-                                                  &outputRightDiffList);
+    if (ignoreWhitespace()) {
+        const QList<Diff> leftIntermediate
+                = Differ::moveWhitespaceIntoEqualities(leftDiffList);
+        const QList<Diff> rightIntermediate
+                = Differ::moveWhitespaceIntoEqualities(rightDiffList);
+        Differ::ignoreWhitespaceBetweenEqualities(leftIntermediate, rightIntermediate,
+                                                  &outputLeftDiffList, &outputRightDiffList);
     } else {
         outputLeftDiffList = leftDiffList;
         outputRightDiffList = rightDiffList;
@@ -122,48 +118,32 @@ void SimpleDiffEditorReloader::reload()
 
     const ChunkData chunkData = DiffUtils::calculateOriginalData(
                 outputLeftDiffList, outputRightDiffList);
-    FileData fileData = DiffUtils::calculateContextData(
-                chunkData, controller()->contextLinesNumber(), 0);
+    FileData fileData = DiffUtils::calculateContextData(chunkData, contextLineCount(), 0);
     fileData.leftFileInfo.fileName = m_leftFileName;
     fileData.rightFileInfo.fileName = m_rightFileName;
 
     QList<FileData> fileDataList;
     fileDataList << fileData;
 
-    controller()->requestSaveState();
-    controller()->setDiffFiles(fileDataList);
-    controller()->requestRestoreState();
-
-    reloadFinished();
+    setDiffFiles(fileDataList);
+    reloadFinished(true);
 }
 
 /////////////////
 
-DiffEditorPlugin::DiffEditorPlugin()
-{
-}
-
-DiffEditorPlugin::~DiffEditorPlugin()
-{
-}
-
 bool DiffEditorPlugin::initialize(const QStringList &arguments, QString *errorMessage)
 {
     Q_UNUSED(arguments)
     Q_UNUSED(errorMessage)
 
     //register actions
-    Core::ActionContainer *toolsContainer =
-        Core::ActionManager::actionContainer(Core::Constants::M_TOOLS);
-    toolsContainer->insertGroup(Core::Constants::G_TOOLS_OPTIONS,
-                                Constants::G_TOOLS_DIFF);
-
-    Core::Context globalcontext(Core::Constants::C_GLOBAL);
+    Core::ActionContainer *toolsContainer
+            = Core::ActionManager::actionContainer(Core::Constants::M_TOOLS);
+    toolsContainer->insertGroup(Core::Constants::G_TOOLS_OPTIONS, Constants::G_TOOLS_DIFF);
 
     QAction *diffAction = new QAction(tr("Diff..."), this);
-    Core::Command *diffCommand = Core::ActionManager::registerAction(diffAction,
-                             "DiffEditor.Diff", globalcontext);
-    connect(diffAction, SIGNAL(triggered()), this, SLOT(diff()));
+    Core::Command *diffCommand = Core::ActionManager::registerAction(diffAction, "DiffEditor.Diff");
+    connect(diffAction, &QAction::triggered, this, &DiffEditorPlugin::diff);
     toolsContainer->addAction(diffCommand, Constants::G_TOOLS_DIFF);
 
     addAutoReleasedObject(new DiffEditorFactory(this));
@@ -174,8 +154,7 @@ bool DiffEditorPlugin::initialize(const QStringList &arguments, QString *errorMe
 }
 
 void DiffEditorPlugin::extensionsInitialized()
-{
-}
+{ }
 
 void DiffEditorPlugin::diff()
 {
@@ -192,23 +171,19 @@ void DiffEditorPlugin::diff()
         return;
 
 
-    const QString documentId = QLatin1String("Diff ") + fileName1
-            + QLatin1String(", ") + fileName2;
+    const QString documentId = QLatin1String("Diff ") + fileName1 + QLatin1String(", ") + fileName2;
     QString title = tr("Diff \"%1\", \"%2\"").arg(fileName1).arg(fileName2);
-    DiffEditorDocument * const document = DiffEditorManager::findOrCreate(documentId, title);
+    auto const document
+            = qobject_cast<DiffEditorDocument *>(DiffEditorManager::findOrCreate(documentId, title));
     if (!document)
         return;
 
-    DiffEditorController *controller = document->controller();
-    if (!controller->reloader()) {
-        SimpleDiffEditorReloader *reloader =
-                new SimpleDiffEditorReloader(fileName1, fileName2);
-        controller->setReloader(reloader);
-    }
-
+    DiffEditorController *controller = DiffEditorManager::controller(document);
+    if (!controller)
+        controller = new FileDiffController(document, fileName1, fileName2);
+    QTC_ASSERT(controller, return);
     Core::EditorManager::activateEditorForDocument(document);
-
-    document->controller()->requestReload();
+    document->reload();
 }
 
 } // namespace Internal
@@ -893,6 +868,66 @@ void DiffEditor::Internal::DiffEditorPlugin::testReadPatch_data()
     fileDataList7 << fileData1;
     QTest::newRow("Dirty submodule") << patch
                                      << fileDataList7;
+
+    //////////////
+
+    // Subversion New
+    patch = _("Index: src/plugins/subversion/subversioneditor.cpp\n"
+              "===================================================================\n"
+              "--- src/plugins/subversion/subversioneditor.cpp\t(revision 0)\n"
+              "+++ src/plugins/subversion/subversioneditor.cpp\t(revision 0)\n"
+              "@@ -0,0 +125 @@\n\n");
+    fileData1 = FileData();
+    fileData1.leftFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    fileData1.rightFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    chunkData1 = ChunkData();
+    chunkData1.leftStartingLineNumber = -1;
+    chunkData1.rightStartingLineNumber = 124;
+    fileData1.chunks << chunkData1;
+    QList<FileData> fileDataList8;
+    fileDataList8 << fileData1;
+    QTest::newRow("Subversion New") << patch
+                                    << fileDataList8;
+
+    //////////////
+
+    // Subversion Deleted
+    patch = _("Index: src/plugins/subversion/subversioneditor.cpp\n"
+              "===================================================================\n"
+              "--- src/plugins/subversion/subversioneditor.cpp\t(revision 42)\n"
+              "+++ src/plugins/subversion/subversioneditor.cpp\t(working copy)\n"
+              "@@ -1,125 +0,0 @@\n\n");
+    fileData1 = FileData();
+    fileData1.leftFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    fileData1.rightFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    chunkData1 = ChunkData();
+    chunkData1.leftStartingLineNumber = 0;
+    chunkData1.rightStartingLineNumber = -1;
+    fileData1.chunks << chunkData1;
+    QList<FileData> fileDataList9;
+    fileDataList9 << fileData1;
+    QTest::newRow("Subversion Deleted") << patch
+                                        << fileDataList9;
+
+    //////////////
+
+    // Subversion Normal
+    patch = _("Index: src/plugins/subversion/subversioneditor.cpp\n"
+              "===================================================================\n"
+              "--- src/plugins/subversion/subversioneditor.cpp\t(revision 42)\n"
+              "+++ src/plugins/subversion/subversioneditor.cpp\t(working copy)\n"
+              "@@ -120,7 +120,7 @@\n\n");
+    fileData1 = FileData();
+    fileData1.leftFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    fileData1.rightFileInfo = DiffFileInfo(_("src/plugins/subversion/subversioneditor.cpp"));
+    chunkData1 = ChunkData();
+    chunkData1.leftStartingLineNumber = 119;
+    chunkData1.rightStartingLineNumber = 119;
+    fileData1.chunks << chunkData1;
+    QList<FileData> fileDataList10;
+    fileDataList10 << fileData1;
+    QTest::newRow("Subversion Normal") << patch
+                                       << fileDataList10;
 }
 
 void DiffEditor::Internal::DiffEditorPlugin::testReadPatch()
