@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -115,13 +115,12 @@ ISettingsAspect *ISettingsAspect::clone() const
 //
 ///////////////////////////////////////////////////////////////////////
 
-IRunConfigurationAspect::IRunConfigurationAspect(RunConfiguration *parent)
+IRunConfigurationAspect::IRunConfigurationAspect(RunConfiguration *runConfig)
 {
-    m_runConfiguration = parent;
+    m_runConfiguration = runConfig;
     m_projectSettings = 0;
     m_globalSettings = 0;
     m_useGlobalSettings = false;
-    connect(this, SIGNAL(requestRunActionsUpdate()), parent, SIGNAL(requestRunActionsUpdate()));
 }
 
 IRunConfigurationAspect::~IRunConfigurationAspect()
@@ -171,9 +170,9 @@ void IRunConfigurationAspect::toMap(QVariantMap &map) const
     map.insert(m_id.toString() + QLatin1String(".UseGlobalSettings"), m_useGlobalSettings);
 }
 
-IRunConfigurationAspect *IRunConfigurationAspect::clone(RunConfiguration *parent) const
+IRunConfigurationAspect *IRunConfigurationAspect::clone(RunConfiguration *runConfig) const
 {
-    IRunConfigurationAspect *other = create(parent);
+    IRunConfigurationAspect *other = create(runConfig);
     if (m_projectSettings)
         other->m_projectSettings = m_projectSettings->clone();
     other->m_globalSettings = m_globalSettings;
@@ -253,6 +252,14 @@ void RunConfiguration::addExtraAspect(IRunConfigurationAspect *aspect)
 void RunConfiguration::ctor()
 {
     connect(this, SIGNAL(enabledChanged()), this, SIGNAL(requestRunActionsUpdate()));
+
+    Utils::MacroExpander *expander = macroExpander();
+    expander->setDisplayName(tr("Run Settings"));
+    expander->setAccumulating(true);
+    expander->registerSubProvider([this]() -> Utils::MacroExpander * {
+        BuildConfiguration *bc = target()->activeBuildConfiguration();
+        return bc ? bc->macroExpander() : target()->macroExpander();
+    });
 }
 
 /*!
@@ -306,12 +313,12 @@ QVariantMap RunConfiguration::toMap() const
     return map;
 }
 
-ProjectExplorer::Abi RunConfiguration::abi() const
+Abi RunConfiguration::abi() const
 {
     BuildConfiguration *bc = target()->activeBuildConfiguration();
     if (!bc)
         return Abi::hostAbi();
-    ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+    ToolChain *tc = ToolChainKitInformation::toolChain(target()->kit());
     if (!tc)
         return Abi::hostAbi();
     return tc->targetAbi();
