@@ -49,6 +49,7 @@
 #include <qtsupport/debugginghelperbuildtask.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
+#include <qtsupport/qtsupportconstants.h>
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcprocess.h>
@@ -58,6 +59,7 @@
 
 using namespace QmakeProjectManager;
 using namespace QmakeProjectManager::Internal;
+using namespace QtSupport;
 using namespace ProjectExplorer;
 using namespace Utils;
 
@@ -67,6 +69,7 @@ const char QMAKE_BS_ID[] = "QtProjectManager.QMakeBuildStep";
 const char QMAKE_ARGUMENTS_KEY[] = "QtProjectManager.QMakeBuildStep.QMakeArguments";
 const char QMAKE_FORCED_KEY[] = "QtProjectManager.QMakeBuildStep.QMakeForced";
 const char QMAKE_USE_QTQUICKCOMPILER[] = "QtProjectManager.QMakeBuildStep.UseQtQuickCompiler";
+const char QMAKE_SEPARATEDEBUGINFO_KEY[] = "QtProjectManager.QMakeBuildStep.SeparateDebugInfo";
 const char QMAKE_QMLDEBUGLIBAUTO_KEY[] = "QtProjectManager.QMakeBuildStep.LinkQmlDebuggingLibraryAuto";
 const char QMAKE_QMLDEBUGLIB_KEY[] = "QtProjectManager.QMakeBuildStep.LinkQmlDebuggingLibrary";
 }
@@ -76,7 +79,8 @@ QMakeStep::QMakeStep(BuildStepList *bsl) :
     m_forced(false),
     m_needToRunQMake(false),
     m_linkQmlDebuggingLibrary(DebugLink),
-    m_useQtQuickCompiler(false)
+    m_useQtQuickCompiler(false),
+    m_separateDebugInfo(false)
 {
     ctor();
 }
@@ -85,7 +89,8 @@ QMakeStep::QMakeStep(BuildStepList *bsl, Core::Id id) :
     AbstractProcessStep(bsl, id),
     m_forced(false),
     m_linkQmlDebuggingLibrary(DebugLink),
-    m_useQtQuickCompiler(false)
+    m_useQtQuickCompiler(false),
+    m_separateDebugInfo(false)
 {
     ctor();
 }
@@ -95,7 +100,8 @@ QMakeStep::QMakeStep(BuildStepList *bsl, QMakeStep *bs) :
     m_forced(bs->m_forced),
     m_userArgs(bs->m_userArgs),
     m_linkQmlDebuggingLibrary(bs->m_linkQmlDebuggingLibrary),
-    m_useQtQuickCompiler(bs->m_useQtQuickCompiler)
+    m_useQtQuickCompiler(bs->m_useQtQuickCompiler),
+    m_separateDebugInfo(bs->m_separateDebugInfo)
 {
     ctor();
 }
@@ -127,9 +133,9 @@ QString QMakeStep::allArguments(bool shorted)
     QmakeBuildConfiguration *bc = qmakeBuildConfiguration();
     QStringList arguments;
     if (bc->subNodeBuild())
-        arguments << QDir::toNativeSeparators(bc->subNodeBuild()->path());
+        arguments << bc->subNodeBuild()->path().toUserOutput();
     else if (shorted)
-        arguments << project()->projectFilePath().toFileInfo().fileName();
+        arguments << project()->projectFilePath().fileName();
     else
         arguments << project()->projectFilePath().toUserOutput();
 
@@ -150,64 +156,43 @@ QString QMakeStep::allArguments(bool shorted)
     // Find out what flags we pass on to qmake
     arguments << bc->configCommandLineArguments();
 
-    arguments << deducedArguments();
+    arguments << deducedArguments().toArguments();
 
     QString args = QtcProcess::joinArgs(arguments);
     // User arguments
     QtcProcess::addArgs(&args, m_userArgs);
-    // moreArgumentsAfter
-    foreach (const QString &arg, deducedArgumentsAfter())
-        QtcProcess::addArg(&args, arg);
     return args;
 }
 
-///
-/// moreArguments,
-/// iphoneos/iphonesimulator for ios
-/// QMAKE_VAR_QMLJSDEBUGGER_PATH
-QStringList QMakeStep::deducedArguments()
+QMakeStepConfig QMakeStep::deducedArguments()
 {
-    QStringList arguments;
+    ProjectExplorer::Kit *kit = target()->kit();
+    QMakeStepConfig config;
     ProjectExplorer::ToolChain *tc
-            = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+            = ProjectExplorer::ToolChainKitInformation::toolChain(kit);
     ProjectExplorer::Abi targetAbi;
     if (tc)
         targetAbi = tc->targetAbi();
 
-    // explicitly add architecture to CONFIG
-
     QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
-    arguments << QmakeBuildConfiguration::deduceArgumnetsForTargetAbi(targetAbi, version);
-    if (linkQmlDebuggingLibrary() && version && !useQtQuickCompiler()) {
-        arguments << QLatin1String(Constants::QMAKEVAR_QUICK1_DEBUG);
+
+    config.archConfig = QMakeStepConfig::targetArchFor(targetAbi, version);
+    config.osType = QMakeStepConfig::osTypeFor(targetAbi, version);
+    if (linkQmlDebuggingLibrary() && version) {
+        config.linkQmlDebuggingQQ1 = true;
         if (version->qtVersion().majorVersion >= 5)
-            arguments << QLatin1String(Constants::QMAKEVAR_QUICK2_DEBUG);
+            config.linkQmlDebuggingQQ2 = true;
     }
 
     if (useQtQuickCompiler() && version)
-        arguments << QLatin1String("CONFIG+=qtquickcompiler");
+        config.useQtQuickCompiler = true;
 
-    return arguments;
+    if (separateDebugInfo())
+        config.separateDebugInfo = true;
+
+    return config;
 }
 
-/// -after OBJECTS_DIR, MOC_DIR, UI_DIR, RCC_DIR
-QStringList QMakeStep::deducedArgumentsAfter()
-{
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
-    if (version && !version->supportsShadowBuilds()) {
-        // We have a target which does not allow shadow building.
-        // But we really don't want to have the build artefacts in the source dir
-        // so we try to hack around it, to make the common cases work.
-        // This is a HACK, remove once all make generators support
-        // shadow building
-        return QStringList() << QLatin1String("-after")
-                             << QLatin1String("OBJECTS_DIR=obj")
-                             << QLatin1String("MOC_DIR=moc")
-                             << QLatin1String("UI_DIR=ui")
-                             << QLatin1String("RCC_DIR=rcc");
-    }
-    return QStringList();
-}
 
 bool QMakeStep::init()
 {
@@ -260,7 +245,7 @@ bool QMakeStep::init()
     QmakeProFileNode *node = static_cast<QmakeProject *>(qt4bc->target()->project())->rootQmakeProjectNode();
     if (qt4bc->subNodeBuild())
         node = qt4bc->subNodeBuild();
-    QString proFile = node->path();
+    QString proFile = node->path().toString();
 
     QList<ProjectExplorer::Task> tasks = qtVersion->reportIssues(proFile, workingDirectory);
     Utils::sort(tasks);
@@ -393,6 +378,23 @@ void QMakeStep::setUseQtQuickCompiler(bool enable)
     qmakeBuildConfiguration()->emitProFileEvaluateNeeded();
 }
 
+bool QMakeStep::separateDebugInfo() const
+{
+    return m_separateDebugInfo;
+}
+
+void QMakeStep::setSeparateDebugInfo(bool enable)
+{
+    if (enable == m_separateDebugInfo)
+        return;
+    m_separateDebugInfo = enable;
+
+    emit separateDebugInfoChanged();
+
+    qmakeBuildConfiguration()->emitQMakeBuildConfigurationChanged();
+    qmakeBuildConfiguration()->emitProFileEvaluateNeeded();
+}
+
 QStringList QMakeStep::parserArguments()
 {
     QStringList result;
@@ -428,6 +430,7 @@ QVariantMap QMakeStep::toMap() const
     map.insert(QLatin1String(QMAKE_QMLDEBUGLIB_KEY), m_linkQmlDebuggingLibrary == DoLink);
     map.insert(QLatin1String(QMAKE_FORCED_KEY), m_forced);
     map.insert(QLatin1String(QMAKE_USE_QTQUICKCOMPILER), m_useQtQuickCompiler);
+    map.insert(QLatin1String(QMAKE_SEPARATEDEBUGINFO_KEY), m_separateDebugInfo);
     return map;
 }
 
@@ -444,6 +447,7 @@ bool QMakeStep::fromMap(const QVariantMap &map)
         else
             m_linkQmlDebuggingLibrary = DoNotLink;
     }
+    m_separateDebugInfo = map.value(QLatin1String(QMAKE_SEPARATEDEBUGINFO_KEY), false).toBool();
 
     return BuildStep::fromMap(map);
 }
@@ -461,6 +465,7 @@ QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
     m_ui->qmakeAdditonalArgumentsLineEdit->setText(m_step->userArguments());
     m_ui->qmlDebuggingLibraryCheckBox->setChecked(m_step->linkQmlDebuggingLibrary());
     m_ui->qtQuickCompilerCheckBox->setChecked(m_step->useQtQuickCompiler());
+    m_ui->separateDebugInfoCheckBox->setChecked(m_step->separateDebugInfo());
 
     qmakeBuildConfigChanged();
 
@@ -481,6 +486,10 @@ QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
             this, &QMakeStepConfigWidget::useQtQuickCompilerChecked);
     connect(m_ui->qtQuickCompilerCheckBox, &QCheckBox::clicked,
             this, &QMakeStepConfigWidget::askForRebuild);
+    connect(m_ui->separateDebugInfoCheckBox, &QAbstractButton::toggled,
+            this, &QMakeStepConfigWidget::separateDebugInfoChecked);
+    connect(m_ui->separateDebugInfoCheckBox, &QCheckBox::clicked,
+            this, &QMakeStepConfigWidget::askForRebuild);
     connect(step, SIGNAL(userArgumentsChanged()),
             this, SLOT(userArgumentsChanged()));
     connect(step, SIGNAL(linkQmlDebuggingLibraryChanged()),
@@ -489,6 +498,8 @@ QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
             this, &QMakeStepConfigWidget::linkQmlDebuggingLibraryChanged);
     connect(step, &QMakeStep::useQtQuickCompilerChanged,
             this, &QMakeStepConfigWidget::useQtQuickCompilerChanged);
+    connect(step, &QMakeStep::separateDebugInfoChanged,
+            this, &QMakeStepConfigWidget::separateDebugInfoChanged);
     connect(step->qmakeBuildConfiguration(), SIGNAL(qmakeBuildConfigurationChanged()),
             this, SLOT(qmakeBuildConfigChanged()));
     connect(step->target(), SIGNAL(kitChanged()), this, SLOT(qtVersionChanged()));
@@ -566,6 +577,15 @@ void QMakeStepConfigWidget::useQtQuickCompilerChanged()
     updateQmlDebuggingOption();
 }
 
+void QMakeStepConfigWidget::separateDebugInfoChanged()
+{
+    if (m_ignoreChange)
+        return;
+
+    updateSummaryLabel();
+    updateEffectiveQMakeCall();
+}
+
 void QMakeStepConfigWidget::qmakeArgumentsLineEdited()
 {
     m_ignoreChange = true;
@@ -635,6 +655,19 @@ void QMakeStepConfigWidget::useQtQuickCompilerChecked(bool checked)
     updateQtQuickCompilerOption();
 }
 
+void QMakeStepConfigWidget::separateDebugInfoChecked(bool checked)
+{
+    if (m_ignoreChange)
+        return;
+
+    m_ignoreChange = true;
+    m_step->setSeparateDebugInfo(checked);
+    m_ignoreChange = false;
+
+    updateSummaryLabel();
+    updateEffectiveQMakeCall();
+}
+
 void QMakeStepConfigWidget::updateSummaryLabel()
 {
     QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(m_step->target()->kit());
@@ -645,7 +678,7 @@ void QMakeStepConfigWidget::updateSummaryLabel()
     // We don't want the full path to the .pro file
     QString args = m_step->allArguments(true);
     // And we only use the .pro filename not the full path
-    QString program = qtVersion->qmakeCommand().toFileInfo().fileName();
+    QString program = qtVersion->qmakeCommand().fileName();
     setSummaryText(tr("<b>qmake:</b> %1 %2").arg(program, args));
 }
 
@@ -653,17 +686,18 @@ void QMakeStepConfigWidget::updateQmlDebuggingOption()
 {
     QString warningText;
     bool supported = QtSupport::BaseQtVersion::isQmlDebuggingSupported(m_step->target()->kit(),
-                                                                     &warningText)
-            && !m_step->useQtQuickCompiler();
+                                                                       &warningText);
 
     m_ui->qmlDebuggingLibraryCheckBox->setEnabled(supported);
-    m_ui->debuggingLibraryLabel->setText(tr("Enable QML debugging:"));
+    m_ui->debuggingLibraryLabel->setText(tr("Enable QML debugging and profiling:"));
 
     if (supported && m_step->linkQmlDebuggingLibrary())
         warningText = tr("Might make your application vulnerable. Only use in a safe environment.");
 
     m_ui->qmlDebuggingWarningText->setText(warningText);
     m_ui->qmlDebuggingWarningIcon->setVisible(!warningText.isEmpty());
+
+    updateQtQuickCompilerOption(); // show or clear compiler warning text
 }
 
 void QMakeStepConfigWidget::updateQtQuickCompilerOption()
@@ -673,6 +707,10 @@ void QMakeStepConfigWidget::updateQtQuickCompilerOption()
                                                                           &warningText);
     m_ui->qtQuickCompilerCheckBox->setEnabled(supported);
     m_ui->qtQuickCompilerLabel->setText(tr("Enable Qt Quick Compiler:"));
+
+    if (supported && m_step->useQtQuickCompiler() && m_step->linkQmlDebuggingLibrary())
+        warningText = tr("Disables QML debugging. QML profiling will still work.");
+
     m_ui->qtQuickCompilerWarningText->setText(warningText);
     m_ui->qtQuickCompilerWarningIcon->setVisible(!warningText.isEmpty());
 }
@@ -682,7 +720,7 @@ void QMakeStepConfigWidget::updateEffectiveQMakeCall()
     QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(m_step->target()->kit());
     QString program = tr("<No Qt version>");
     if (qtVersion)
-        program = qtVersion->qmakeCommand().toFileInfo().fileName();
+        program = qtVersion->qmakeCommand().fileName();
     m_ui->qmakeArgumentsEdit->setPlainText(program + QLatin1Char(' ') + m_step->allArguments());
 }
 
@@ -782,3 +820,76 @@ QString QMakeStepFactory::displayNameForId(Core::Id id) const
         return tr("qmake");
     return QString();
 }
+
+
+QMakeStepConfig::TargetArchConfig QMakeStepConfig::targetArchFor(const Abi &targetAbi, const BaseQtVersion *version)
+{
+    QMakeStepConfig::TargetArchConfig arch = QMakeStepConfig::NoArch;
+    if (!version || version->type() != QLatin1String(QtSupport::Constants::DESKTOPQT))
+        return arch;
+    if ((targetAbi.os() == ProjectExplorer::Abi::MacOS)
+            && (targetAbi.binaryFormat() == ProjectExplorer::Abi::MachOFormat)) {
+        if (targetAbi.architecture() == ProjectExplorer::Abi::X86Architecture) {
+            if (targetAbi.wordWidth() == 32)
+                arch = QMakeStepConfig::X86;
+            else if (targetAbi.wordWidth() == 64)
+                arch = QMakeStepConfig::X86_64;
+        } else if (targetAbi.architecture() == ProjectExplorer::Abi::PowerPCArchitecture) {
+            if (targetAbi.wordWidth() == 32)
+                arch = QMakeStepConfig::PPC;
+            else if (targetAbi.wordWidth() == 64)
+                arch = QMakeStepConfig::PPC64;
+        }
+    }
+    return arch;
+}
+
+QMakeStepConfig::OsType QMakeStepConfig::osTypeFor(const ProjectExplorer::Abi &targetAbi, const BaseQtVersion *version)
+{
+    QMakeStepConfig::OsType os = QMakeStepConfig::NoOsType;
+    const char IOSQT[] = "Qt4ProjectManager.QtVersion.Ios";
+    if (!version || version->type() != QLatin1String(IOSQT))
+        return os;
+    if ((targetAbi.os() == ProjectExplorer::Abi::MacOS)
+            && (targetAbi.binaryFormat() == ProjectExplorer::Abi::MachOFormat)) {
+        if (targetAbi.architecture() == ProjectExplorer::Abi::X86Architecture) {
+            os = QMakeStepConfig::IphoneSimulator;
+        } else if (targetAbi.architecture() == ProjectExplorer::Abi::ArmArchitecture) {
+            os = QMakeStepConfig::IphoneOS;
+        }
+    }
+    return os;
+}
+
+QStringList QMakeStepConfig::toArguments() const
+{
+    QStringList arguments;
+    if (archConfig == X86)
+        arguments << QLatin1String("CONFIG+=x86");
+    else if (archConfig == X86_64)
+        arguments << QLatin1String("CONFIG+=x86_64");
+    else if (archConfig == PPC)
+        arguments << QLatin1String("CONFIG+=ppc");
+    else if (archConfig == PPC64)
+        arguments << QLatin1String("CONFIG+=ppc64");
+
+    if (osType == IphoneSimulator)
+        arguments << QLatin1String("CONFIG+=iphonesimulator");
+    else if (osType == IphoneOS)
+        arguments << QLatin1String("CONFIG+=iphoneos");
+
+    if (linkQmlDebuggingQQ1)
+        arguments << QLatin1String("CONFIG+=declarative_debug");
+    if (linkQmlDebuggingQQ2)
+        arguments << QLatin1String("CONFIG+=qml_debug");
+
+    if (useQtQuickCompiler)
+        arguments << QLatin1String("CONFIG+=qtquickcompiler");
+
+    if (separateDebugInfo)
+        arguments << QLatin1String("CONFIG+=force_debug_info")
+                  << QLatin1String("CONFIG+=separate_debug_info");
+
+    return arguments;
+}
+
