@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -43,17 +44,34 @@ using namespace QmlDebug;
 
 const char PROFILER_FILE_VERSION[] = "1.02";
 
-const char TYPE_PAINTING_STR[] = "Painting";
-const char TYPE_COMPILING_STR[] = "Compiling";
-const char TYPE_CREATING_STR[] = "Creating";
-const char TYPE_BINDING_STR[] = "Binding";
-const char TYPE_HANDLINGSIGNAL_STR[] = "HandlingSignal";
-const char TYPE_PIXMAPCACHE_STR[] = "PixmapCache";
-const char TYPE_SCENEGRAPH_STR[] = "SceneGraph";
+static const char *RANGE_TYPE_STRINGS[] = {
+    "Painting",
+    "Compiling",
+    "Creating",
+    "Binding",
+    "HandlingSignal",
+    "Javascript"
+};
+
+Q_STATIC_ASSERT(sizeof(RANGE_TYPE_STRINGS) == QmlDebug::MaximumRangeType * sizeof(const char *));
+
+static const char *MESSAGE_STRINGS[] = {
+    // So far only pixmap and scenegraph are used. The others are padding.
+    "Event",
+    "RangeStart",
+    "RangeData",
+    "RangeLocation",
+    "RangeEnd",
+    "Complete",
+    "PixmapCache",
+    "SceneGraph",
+    "MemoryAllocation"
+};
+
+Q_STATIC_ASSERT(sizeof(MESSAGE_STRINGS) == QmlDebug::MaximumMessage * sizeof(const char *));
 
 #define _(X) QLatin1String(X)
 
-
 //
 // "be strict in your output but tolerant in your inputs"
 //
@@ -61,59 +79,44 @@ const char TYPE_SCENEGRAPH_STR[] = "SceneGraph";
 namespace QmlProfiler {
 namespace Internal {
 
-static QmlEventType qmlEventTypeAsEnum(const QString &typeString)
+static QPair<Message, RangeType> qmlTypeAsEnum(const QString &typeString)
 {
-    if (typeString == _(TYPE_PAINTING_STR)) {
-        return Painting;
-    } else if (typeString == _(TYPE_COMPILING_STR)) {
-        return Compiling;
-    } else if (typeString == _(TYPE_CREATING_STR)) {
-        return Creating;
-    } else if (typeString == _(TYPE_BINDING_STR)) {
-        return Binding;
-    } else if (typeString == _(TYPE_HANDLINGSIGNAL_STR)) {
-        return HandlingSignal;
-    } else if (typeString == _(TYPE_PIXMAPCACHE_STR)) {
-        return PixmapCacheEvent;
-    } else if (typeString == _(TYPE_SCENEGRAPH_STR)) {
-        return SceneGraphFrameEvent;
-    } else {
+    QPair<Message, RangeType> ret(MaximumMessage,
+                                                      MaximumRangeType);
+
+    for (int i = 0; i < MaximumMessage; ++i) {
+        if (typeString == _(MESSAGE_STRINGS[i])) {
+            ret.first = static_cast<Message>(i);
+            break;
+        }
+    }
+
+    for (int i = 0; i < MaximumRangeType; ++i) {
+        if (typeString == _(RANGE_TYPE_STRINGS[i])) {
+            ret.second = static_cast<RangeType>(i);
+            break;
+        }
+    }
+
+    if (ret.first == MaximumMessage && ret.second == MaximumRangeType) {
         bool isNumber = false;
         int type = typeString.toUInt(&isNumber);
-        if (isNumber)
-            return (QmlEventType)type;
-        else
-            return MaximumQmlEventType;
+        if (isNumber && type < MaximumRangeType)
+            // Allow saving ranges as numbers, but not messages.
+            ret.second = static_cast<RangeType>(type);
     }
+
+    return ret;
 }
 
-static QString qmlEventTypeAsString(QmlEventType typeEnum)
+static QString qmlTypeAsString(Message message, RangeType rangeType)
 {
-    switch (typeEnum) {
-    case Painting:
-        return _(TYPE_PAINTING_STR);
-        break;
-    case Compiling:
-        return _(TYPE_COMPILING_STR);
-        break;
-    case Creating:
-        return _(TYPE_CREATING_STR);
-        break;
-    case Binding:
-        return _(TYPE_BINDING_STR);
-        break;
-    case HandlingSignal:
-        return _(TYPE_HANDLINGSIGNAL_STR);
-        break;
-    case PixmapCacheEvent:
-        return _(TYPE_PIXMAPCACHE_STR);
-        break;
-    case SceneGraphFrameEvent:
-        return _(TYPE_SCENEGRAPH_STR);
-        break;
-    default:
-        return QString::number((int)typeEnum);
-    }
+    if (rangeType < MaximumRangeType)
+        return _(RANGE_TYPE_STRINGS[rangeType]);
+    else if (message != MaximumMessage)
+        return _(MESSAGE_STRINGS[message]);
+    else
+        return QString::number((int)rangeType);
 }
 
 
@@ -128,11 +131,18 @@ void QmlProfilerFileReader::setV8DataModel(QV8ProfilerDataModel *dataModel)
     m_v8Model = dataModel;
 }
 
+void QmlProfilerFileReader::setQmlDataModel(QmlProfilerDataModel *dataModel)
+{
+    m_qmlModel = dataModel;
+}
+
 bool QmlProfilerFileReader::load(QIODevice *device)
 {
     QXmlStreamReader stream(device);
 
     bool validVersion = true;
+    qint64 traceStart = -1;
+    qint64 traceEnd = -1;
 
     while (validVersion && !stream.atEnd() && !stream.hasError()) {
          QXmlStreamReader::TokenType token = stream.readNext();
@@ -147,9 +157,9 @@ bool QmlProfilerFileReader::load(QIODevice *device)
                  else
                      validVersion = false;
                  if (attributes.hasAttribute(_("traceStart")))
-                     emit traceStartTime(attributes.value(_("traceStart")).toString().toLongLong());
+                     traceStart = attributes.value(_("traceStart")).toString().toLongLong();
                  if (attributes.hasAttribute(_("traceEnd")))
-                     emit traceEndTime(attributes.value(_("traceEnd")).toString().toLongLong());
+                     traceEnd = attributes.value(_("traceEnd")).toString().toLongLong();
              }
 
              if (elementName == _("eventData")) {
@@ -162,6 +172,11 @@ bool QmlProfilerFileReader::load(QIODevice *device)
                  break;
              }
 
+             if (elementName == _("noteData")) {
+                 loadNoteData(stream);
+                 break;
+             }
+
              if (elementName == _("v8profile")) {
                  if (m_v8Model)
                      m_v8Model->load(stream);
@@ -178,8 +193,8 @@ bool QmlProfilerFileReader::load(QIODevice *device)
          emit error(tr("Error while parsing trace data file: %1").arg(stream.errorString()));
          return false;
      } else {
-         processQmlEvents();
-
+         m_qmlModel->setData(traceStart, qMax(traceStart, traceEnd), m_qmlEvents, m_ranges);
+         m_qmlModel->setNoteData(m_notes);
          return true;
      }
 }
@@ -191,16 +206,15 @@ void QmlProfilerFileReader::loadEventData(QXmlStreamReader &stream)
     QXmlStreamAttributes attributes = stream.attributes();
 
     int eventIndex = -1;
-    QmlEvent event = {
+    QmlProfilerDataModel::QmlEventTypeData event = {
             QString(), // displayname
-            QString(), // filename
-            QString(), // details
+            QmlEventLocation(),
+            MaximumMessage,
             Painting, // type
             QmlBinding,  // bindingType, set for backwards compatibility
-            0, // line
-            0 // column
+            QString(), // details
     };
-    const QmlEvent defaultEvent = event;
+    const QmlProfilerDataModel::QmlEventTypeData defaultEvent = event;
 
     while (!stream.atEnd() && !stream.hasError()) {
         QXmlStreamReader::TokenType token = stream.readNext();
@@ -233,35 +247,52 @@ void QmlProfilerFileReader::loadEventData(QXmlStreamReader &stream)
             }
 
             if (elementName == _("type")) {
-                event.type = qmlEventTypeAsEnum(readData);
+                QPair<Message, RangeType> enums = qmlTypeAsEnum(readData);
+                event.message = enums.first;
+                event.rangeType = enums.second;
                 break;
             }
 
             if (elementName == _("filename")) {
-                event.filename = readData;
+                event.location.filename = readData;
                 break;
             }
 
             if (elementName == _("line")) {
-                event.line = readData.toInt();
+                event.location.line = readData.toInt();
                 break;
             }
 
             if (elementName == _("column")) {
-                event.column = readData.toInt();
+                event.location.column = readData.toInt();
                 break;
             }
 
             if (elementName == _("details")) {
-                event.details = readData;
+                event.data = readData;
                 break;
             }
 
+            if (elementName == _("animationFrame")) {
+                event.detailType = readData.toInt();
+                // new animation frames used to be saved as ranges of range type Painting with
+                // binding type 4 (which was called "AnimationFrame" to make everything even more
+                // confusing), even though they clearly aren't ranges. Convert that to something
+                // sane here.
+                if (event.detailType == 4) {
+                    event.message = Event;
+                    event.rangeType = MaximumRangeType;
+                    event.detailType = AnimationFrame;
+                }
+            }
+
             if (elementName == _("bindingType") ||
-                    elementName == _("animationFrame") ||
                     elementName == _("cacheEventType") ||
-                    elementName == _("sgEventType")) {
-                event.bindingType = readData.toInt();
+                    elementName == _("sgEventType") ||
+                    elementName == _("memoryEventType") ||
+                    elementName == _("mouseEvent") ||
+                    elementName == _("keyEvent")) {
+                event.detailType = readData.toInt();
                 break;
             }
 
@@ -299,7 +330,7 @@ void QmlProfilerFileReader::loadProfilerDataModel(QXmlStreamReader &stream)
         switch (token) {
         case QXmlStreamReader::StartElement: {
             if (elementName == _("range")) {
-                Range range = { 0, 0, 0, 0, 0, 0, 0 };
+                QmlProfilerDataModel::QmlEventData range = { -1, 0, 0, 0, 0, 0, 0, 0 };
 
                 const QXmlStreamAttributes attributes = stream.attributes();
                 if (!attributes.hasAttribute(_("startTime"))
@@ -325,6 +356,8 @@ void QmlProfilerFileReader::loadProfilerDataModel(QXmlStreamReader &stream)
                     range.numericData2 = attributes.value(_("height")).toString().toLongLong();
                 if (attributes.hasAttribute(_("refCount")))
                     range.numericData3 = attributes.value(_("refCount")).toString().toLongLong();
+                if (attributes.hasAttribute(_("amount")))
+                    range.numericData1 = attributes.value(_("amount")).toString().toLongLong();
                 if (attributes.hasAttribute(_("timing1")))
                     range.numericData1 = attributes.value(_("timing1")).toString().toLongLong();
                 if (attributes.hasAttribute(_("timing2")))
@@ -336,11 +369,9 @@ void QmlProfilerFileReader::loadProfilerDataModel(QXmlStreamReader &stream)
                 if (attributes.hasAttribute(_("timing5")))
                     range.numericData5 = attributes.value(_("timing5")).toString().toLongLong();
 
+                range.typeIndex = attributes.value(_("eventIndex")).toString().toInt();
 
-                int eventIndex = attributes.value(_("eventIndex")).toString().toInt();
-
-
-                m_ranges.append(QPair<Range,int>(range, eventIndex));
+                m_ranges.append(range);
             }
             break;
         }
@@ -356,25 +387,38 @@ void QmlProfilerFileReader::loadProfilerDataModel(QXmlStreamReader &stream)
     }
 }
 
-void QmlProfilerFileReader::processQmlEvents()
+void QmlProfilerFileReader::loadNoteData(QXmlStreamReader &stream)
 {
-    for (int i = 0; i < m_ranges.size(); ++i) {
-        Range range = m_ranges[i].first;
-        int eventIndex = m_ranges[i].second;
+    QmlProfilerDataModel::QmlEventNoteData currentNote;
+    while (!stream.atEnd() && !stream.hasError()) {
+        QXmlStreamReader::TokenType token = stream.readNext();
+        const QStringRef elementName = stream.name();
 
-        if (eventIndex < 0 || eventIndex >= m_qmlEvents.size()) {
-            qWarning() << ".qtd file - range index" << eventIndex
-                       << "is outside of bounds (0, " << m_qmlEvents.size() << ")";
-            continue;
+        switch (token) {
+        case QXmlStreamReader::StartElement: {
+            if (elementName == _("note")) {
+                QXmlStreamAttributes attrs = stream.attributes();
+                currentNote.startTime = attrs.value(_("startTime")).toString().toLongLong();
+                currentNote.duration = attrs.value(_("duration")).toString().toLongLong();
+                currentNote.typeIndex = attrs.value(_("eventIndex")).toString().toInt();
+            }
+            break;
+        }
+        case QXmlStreamReader::Characters: {
+            currentNote.text = stream.text().toString();
+            break;
+        }
+        case QXmlStreamReader::EndElement: {
+            if (elementName == _("note")) {
+                m_notes.append(currentNote);
+            } else if (elementName == _("noteData")) {
+                return;
+            }
+            break;
+        }
+        default:
+            break;
         }
-
-        QmlEvent &event = m_qmlEvents[eventIndex];
-
-        emit rangedEvent(event.type, event.bindingType, range.startTime, range.duration,
-                         QStringList(event.details),
-                         QmlEventLocation(event.filename, event.line, event.column),
-                         range.numericData1,range.numericData2, range.numericData3, range.numericData4, range.numericData5);
-
     }
 }
 
@@ -385,7 +429,6 @@ QmlProfilerFileWriter::QmlProfilerFileWriter(QObject *parent) :
     m_measuredTime(0),
     m_v8Model(0)
 {
-    m_acceptedTypes << QmlDebug::Compiling << QmlDebug::Creating << QmlDebug::Binding << QmlDebug::HandlingSignal;
 }
 
 void QmlProfilerFileWriter::setTraceTime(qint64 startTime, qint64 endTime, qint64 measuredTime)
@@ -400,27 +443,16 @@ void QmlProfilerFileWriter::setV8DataModel(QV8ProfilerDataModel *dataModel)
     m_v8Model = dataModel;
 }
 
-void QmlProfilerFileWriter::setQmlEvents(const QVector<QmlProfilerDataModel::QmlEventData> &events)
+void QmlProfilerFileWriter::setQmlEvents(const QVector<QmlProfilerDataModel::QmlEventTypeData> &types,
+                                         const QVector<QmlProfilerDataModel::QmlEventData> &events)
 {
-    foreach (const QmlProfilerDataModel::QmlEventData &event, events) {
-        const QString hashStr = QmlProfilerDataModel::getHashString(event);
-        if (!m_qmlEvents.contains(hashStr)) {
-            QmlEvent e = { event.displayName,
-                           event.location.filename,
-                           event.data.join(_("")),
-                           static_cast<QmlDebug::QmlEventType>(event.eventType),
-                           event.bindingType,
-                           event.location.line,
-                           event.location.column
-                         };
-            m_qmlEvents.insert(hashStr, e);
-        }
+    m_qmlEvents = types;
+    m_ranges = events;
+}
 
-        Range r = { event.startTime, event.duration, event.numericData1, event.numericData2, event.numericData3, event.numericData4, event.numericData5 };
-        m_ranges.append(QPair<Range, QString>(r, hashStr));
-    }
-
-    calculateMeasuredTime(events);
+void QmlProfilerFileWriter::setNotes(const QVector<QmlProfilerDataModel::QmlEventNoteData> &notes)
+{
+    m_notes = notes;
 }
 
 void QmlProfilerFileWriter::save(QIODevice *device)
@@ -439,74 +471,82 @@ void QmlProfilerFileWriter::save(QIODevice *device)
     stream.writeStartElement(_("eventData"));
     stream.writeAttribute(_("totalTime"), QString::number(m_measuredTime));
 
-    QMap<QString, QString> keys;
-    int i = 0;
-    foreach (const QString &key, m_qmlEvents.keys())
-        keys[key] = QString::number(i++);
-
-    QHash<QString,QmlEvent>::const_iterator eventIter = m_qmlEvents.constBegin();
-    for (; eventIter != m_qmlEvents.constEnd(); ++eventIter) {
-        QmlEvent event = eventIter.value();
+     for (int typeIndex = 0; typeIndex < m_qmlEvents.size(); ++typeIndex) {
+        const QmlProfilerDataModel::QmlEventTypeData &event = m_qmlEvents[typeIndex];
 
         stream.writeStartElement(_("event"));
-        stream.writeAttribute(_("index"), keys[eventIter.key()]);
+        stream.writeAttribute(_("index"), QString::number(typeIndex));
         stream.writeTextElement(_("displayname"), event.displayName);
-        stream.writeTextElement(_("type"), qmlEventTypeAsString(event.type));
-        if (!event.filename.isEmpty()) {
-            stream.writeTextElement(_("filename"), event.filename);
-            stream.writeTextElement(_("line"), QString::number(event.line));
-            stream.writeTextElement(_("column"), QString::number(event.column));
+        stream.writeTextElement(_("type"), qmlTypeAsString(event.message, event.rangeType));
+        if (!event.location.filename.isEmpty()) {
+            stream.writeTextElement(_("filename"), event.location.filename);
+            stream.writeTextElement(_("line"), QString::number(event.location.line));
+            stream.writeTextElement(_("column"), QString::number(event.location.column));
+        }
+
+        if (!event.data.isEmpty())
+            stream.writeTextElement(_("details"), event.data);
+
+        if (event.rangeType == Binding) {
+            stream.writeTextElement(_("bindingType"), QString::number(event.detailType));
+        } else if (event.message == Event) {
+            switch (event.detailType) {
+            case AnimationFrame:
+                stream.writeTextElement(_("animationFrame"), QString::number(event.detailType));
+                break;
+            case Key:
+                stream.writeTextElement(_("keyEvent"), QString::number(event.detailType));
+                break;
+            case Mouse:
+                stream.writeTextElement(_("mouseEvent"), QString::number(event.detailType));
+                break;
+            default:
+                break;
+            }
+        } else if (event.message == PixmapCacheEvent) {
+            stream.writeTextElement(_("cacheEventType"), QString::number(event.detailType));
+        } else if (event.message == SceneGraphFrame) {
+            stream.writeTextElement(_("sgEventType"), QString::number(event.detailType));
+        } else if (event.message == MemoryAllocation) {
+            stream.writeTextElement(_("memoryEventType"), QString::number(event.detailType));
         }
-        stream.writeTextElement(_("details"), event.details);
-        if (event.type == Binding)
-            stream.writeTextElement(_("bindingType"), QString::number(event.bindingType));
-        if (event.type == Painting && event.bindingType == AnimationFrame)
-            stream.writeTextElement(_("animationFrame"), QString::number(event.bindingType));
-        if (event.type == PixmapCacheEvent)
-            stream.writeTextElement(_("cacheEventType"), QString::number(event.bindingType));
-        if (event.type == SceneGraphFrameEvent)
-            stream.writeTextElement(_("sgEventType"), QString::number(event.bindingType));
         stream.writeEndElement();
     }
     stream.writeEndElement(); // eventData
 
     stream.writeStartElement(_("profilerDataModel"));
 
-    QVector<QPair<Range, QString> >::const_iterator rangeIter = m_ranges.constBegin();
-    for (; rangeIter != m_ranges.constEnd(); ++rangeIter) {
-        Range range = rangeIter->first;
-        QString eventHash = rangeIter->second;
+    for (int rangeIndex = 0; rangeIndex < m_ranges.size(); ++rangeIndex) {
+        const QmlProfilerDataModel::QmlEventData &range = m_ranges[rangeIndex];
 
         stream.writeStartElement(_("range"));
         stream.writeAttribute(_("startTime"), QString::number(range.startTime));
         if (range.duration > 0) // no need to store duration of instantaneous events
             stream.writeAttribute(_("duration"), QString::number(range.duration));
-        stream.writeAttribute(_("eventIndex"), keys[eventHash]);
+        stream.writeAttribute(_("eventIndex"), QString::number(range.typeIndex));
 
-        QmlEvent event = m_qmlEvents.value(eventHash);
+        const QmlProfilerDataModel::QmlEventTypeData &event = m_qmlEvents[range.typeIndex];
 
         // special: animation event
-        if (event.type == QmlDebug::Painting && event.bindingType == QmlDebug::AnimationFrame) {
-
+        if (event.message == Event && event.detailType == AnimationFrame) {
             stream.writeAttribute(_("framerate"), QString::number(range.numericData1));
             stream.writeAttribute(_("animationcount"), QString::number(range.numericData2));
             stream.writeAttribute(_("thread"), QString::number(range.numericData3));
         }
 
         // special: pixmap cache event
-        if (event.type == QmlDebug::PixmapCacheEvent) {
-            // pixmap image size
-            if (event.bindingType == 0) {
+        if (event.message == PixmapCacheEvent) {
+            if (event.detailType == PixmapSizeKnown) {
                 stream.writeAttribute(_("width"), QString::number(range.numericData1));
                 stream.writeAttribute(_("height"), QString::number(range.numericData2));
             }
 
-            // reference count (1) / cache size changed (2)
-            if (event.bindingType == 1 || event.bindingType == 2)
+            if (event.detailType == PixmapReferenceCountChanged ||
+                    event.detailType == PixmapCacheCountChanged)
                 stream.writeAttribute(_("refCount"), QString::number(range.numericData3));
         }
 
-        if (event.type == QmlDebug::SceneGraphFrameEvent) {
+        if (event.message == SceneGraphFrame) {
             // special: scenegraph frame events
             if (range.numericData1 > 0)
                 stream.writeAttribute(_("timing1"), QString::number(range.numericData1));
@@ -519,47 +559,33 @@ void QmlProfilerFileWriter::save(QIODevice *device)
             if (range.numericData5 > 0)
                 stream.writeAttribute(_("timing5"), QString::number(range.numericData5));
         }
+
+        // special: memory allocation event
+        if (event.message == MemoryAllocation)
+            stream.writeAttribute(_("amount"), QString::number(range.numericData1));
+
         stream.writeEndElement();
     }
     stream.writeEndElement(); // profilerDataModel
 
+    stream.writeStartElement(_("noteData"));
+    for (int noteIndex = 0; noteIndex < m_notes.size(); ++noteIndex) {
+        const QmlProfilerDataModel::QmlEventNoteData &notes = m_notes[noteIndex];
+        stream.writeStartElement(_("note"));
+        stream.writeAttribute(_("startTime"), QString::number(notes.startTime));
+        stream.writeAttribute(_("duration"), QString::number(notes.duration));
+        stream.writeAttribute(_("eventIndex"), QString::number(notes.typeIndex));
+        stream.writeCharacters(notes.text);
+        stream.writeEndElement(); // note
+    }
+    stream.writeEndElement(); // noteData
+
     m_v8Model->save(stream);
 
     stream.writeEndElement(); // trace
     stream.writeEndDocument();
 }
 
-void QmlProfilerFileWriter::calculateMeasuredTime(const QVector<QmlProfilerDataModel::QmlEventData> &events)
-{
-    // measured time isn't used, but old clients might still need it
-    // -> we calculate it explicitly
-
-    qint64 duration = 0;
-
-    QHash<int, qint64> endtimesPerLevel;
-    int level = QmlDebug::Constants::QML_MIN_LEVEL;
-    endtimesPerLevel[0] = 0;
-
-    foreach (const QmlProfilerDataModel::QmlEventData &event, events) {
-        // whitelist
-        if (!m_acceptedTypes.contains(event.eventType))
-            continue;
-
-        // level computation
-        if (endtimesPerLevel[level] > event.startTime) {
-            level++;
-        } else {
-            while (level > QmlDebug::Constants::QML_MIN_LEVEL && endtimesPerLevel[level-1] <= event.startTime)
-                level--;
-        }
-        endtimesPerLevel[level] = event.startTime + event.duration;
-        if (level == QmlDebug::Constants::QML_MIN_LEVEL)
-            duration += event.duration;
-    }
-
-    m_measuredTime = duration;
-}
-
 
 } // namespace Internal
 } // namespace QmlProfiler
