@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -145,11 +145,11 @@ QVariant ModelAdapter::data(const QModelIndex &index, int role) const
 // ------------------------
 // GenericProposalInfoFrame
 // ------------------------
-class GenericProposalInfoFrame : public Utils::FakeToolTip
+class GenericProposalInfoFrame : public FakeToolTip
 {
 public:
     GenericProposalInfoFrame(QWidget *parent = 0)
-        : Utils::FakeToolTip(parent), m_label(new QLabel(this))
+        : FakeToolTip(parent), m_label(new QLabel(this))
     {
         QVBoxLayout *layout = new QVBoxLayout(this);
         layout->setMargin(0);
@@ -531,10 +531,10 @@ void GenericProposalWidget::updatePositionAndSize()
     QPoint pos = d->m_displayRect.bottomLeft();
     pos.rx() -= 16 + fw;    // Space for the icons
     if (pos.y() + height > screen.bottom())
-        pos.setY(d->m_displayRect.top() - height);
+        pos.setY(qMax(0, d->m_displayRect.top() - height));
     if (pos.x() + width > screen.right())
-        pos.setX(screen.right() - width);
-    setGeometry(pos.x(), pos.y(), width, height);
+        pos.setX(qMax(0, screen.right() - width));
+    setGeometry(pos.x(), pos.y(), qMin(width, screen.width()), qMin(height, screen.height()));
 }
 
 void GenericProposalWidget::turnOffAutoWidth()
