@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of the Qt Build Suite.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company. For licensing terms and
+** conditions see http://www.qt.io/terms-conditions. For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -63,24 +63,6 @@ BuildGraphLoader::~BuildGraphLoader()
     qDeleteAll(m_objectsToDelete);
 }
 
-static bool isConfigCompatible(const QVariantMap &cfg1, const QVariantMap &cfg2)
-{
-    if (cfg1.count() != cfg2.count())
-        return false;
-    QVariantMap::const_iterator it = cfg1.begin();
-    for (; it != cfg1.end(); ++it) {
-        if (it.value().type() == QVariant::Map) {
-            if (!isConfigCompatible(it.value().toMap(), cfg2.value(it.key()).toMap()))
-                return false;
-        } else {
-            QVariant value = cfg2.value(it.key());
-            if (value != it.value())
-                return false;
-        }
-    }
-    return true;
-}
-
 static void restoreBackPointers(const ResolvedProjectPtr &project)
 {
     foreach (const ResolvedProductPtr &product, project->products) {
@@ -165,11 +147,11 @@ void BuildGraphLoader::loadBuildGraphFromDisk()
 
 void BuildGraphLoader::checkBuildGraphCompatibility(const TopLevelProjectConstPtr &project)
 {
-    if (QFileInfo(project->location.fileName()) != QFileInfo(m_parameters.projectFilePath())) {
+    if (QFileInfo(project->location.filePath()) != QFileInfo(m_parameters.projectFilePath())) {
         QString errorMessage = Tr::tr("Stored build graph at '%1' is for project file '%2', but "
                                       "input file is '%3'. ")
                 .arg(QDir::toNativeSeparators(project->buildGraphFilePath()),
-                     QDir::toNativeSeparators(project->location.fileName()),
+                     QDir::toNativeSeparators(project->location.filePath()),
                      QDir::toNativeSeparators(m_parameters.projectFilePath()));
         if (!m_parameters.ignoreDifferentProjectFilePath()) {
             errorMessage += Tr::tr("Aborting.");
@@ -182,20 +164,46 @@ void BuildGraphLoader::checkBuildGraphCompatibility(const TopLevelProjectConstPt
     }
 }
 
+static bool checkProductForChangedDependency(QList<ResolvedProductPtr> &changedProducts,
+        QSet<ResolvedProductPtr> &seenProducts, const ResolvedProductPtr &product)
+{
+    if (seenProducts.contains(product))
+        return false;
+    if (changedProducts.contains(product))
+        return true;
+    foreach (const ResolvedProductPtr &dep, product->dependencies) {
+        if (checkProductForChangedDependency(changedProducts, seenProducts, dep)) {
+            changedProducts << product;
+            return true;
+        }
+    }
+    seenProducts << product;
+    return false;
+}
+
+// All products depending on changed products also become changed. Otherwise the output
+// artifacts of the rules taking the artifacts from the dependency as inputs will be
+// rebuilt due to their rule getting re-applied (as the rescued input artifacts will show
+// up as newly added) and no rescue data being available.
+static void makeChangedProductsListComplete(QList<ResolvedProductPtr> &changedProducts,
+                                            const QList<ResolvedProductPtr> &allRestoredProducts)
+{
+    QSet<ResolvedProductPtr> seenProducts;
+    foreach (const ResolvedProductPtr &p, allRestoredProducts)
+        checkProductForChangedDependency(changedProducts, seenProducts, p);
+}
+
 void BuildGraphLoader::trackProjectChanges()
 {
     const TopLevelProjectPtr &restoredProject = m_result.loadedProject;
     QSet<QString> buildSystemFiles = restoredProject->buildSystemFiles;
     QList<ResolvedProductPtr> allRestoredProducts = restoredProject->allProducts();
     QList<ResolvedProductPtr> changedProducts;
-    QList<ResolvedProductPtr> productsWithChangedFiles;
     bool reResolvingNecessary = false;
-    if (!isConfigCompatible(m_parameters.finalBuildConfigurationTree(),
-                            restoredProject->buildConfiguration())) {
+    if (!isConfigCompatible())
         reResolvingNecessary = true;
-    }
     if (hasProductFileChanged(allRestoredProducts, restoredProject->lastResolveTime,
-                              buildSystemFiles, productsWithChangedFiles)) {
+                              buildSystemFiles, changedProducts)) {
         reResolvingNecessary = true;
     }
 
@@ -225,12 +233,11 @@ void BuildGraphLoader::trackProjectChanges()
     foreach (const ResolvedProductPtr &cp, allNewlyResolvedProducts)
         freshProductsByName.insert(cp->uniqueName(), cp);
 
-    checkAllProductsForChanges(allRestoredProducts, freshProductsByName, changedProducts,
-                               productsWithChangedFiles);
+    checkAllProductsForChanges(allRestoredProducts, freshProductsByName, changedProducts);
 
     QSharedPointer<ProjectBuildData> oldBuildData;
     ChildListHash childLists;
-    if (!changedProducts.isEmpty() || !productsWithChangedFiles.isEmpty()) {
+    if (!changedProducts.isEmpty()) {
         oldBuildData = QSharedPointer<ProjectBuildData>(
                     new ProjectBuildData(restoredProject->buildData.data()));
         foreach (const ResolvedProductConstPtr &product, allRestoredProducts) {
@@ -247,9 +254,11 @@ void BuildGraphLoader::trackProjectChanges()
         }
     }
 
-    // For products with "serious" changes such as different prepare scripts, we set up the
-    // build data from scratch to be on the safe side. This can be made more fine-grained
-    // if needed.
+    makeChangedProductsListComplete(changedProducts, allRestoredProducts);
+
+    // Set up build data from scratch for all changed products. This does not necessarily
+    // mean that artifacts will have to get rebuilt; whether this is necesessary will be decided
+    // an a per-artifact basis by the Executor on the next build.
     QHash<QString, AllRescuableArtifactData> rescuableArtifactData;
     foreach (const ResolvedProductPtr &product, changedProducts) {
         ResolvedProductPtr freshProduct = freshProductsByName.value(product->uniqueName());
@@ -261,7 +270,6 @@ void BuildGraphLoader::trackProjectChanges()
                                          product->buildData->rescuableArtifactData);
         }
         allRestoredProducts.removeOne(product);
-        productsWithChangedFiles.removeOne(product);
     }
 
     // Move over restored build data to newly resolved project.
@@ -275,8 +283,6 @@ void BuildGraphLoader::trackProjectChanges()
             if (newlyResolvedProduct->uniqueName() == restoredProduct->uniqueName()) {
                 if (newlyResolvedProduct->enabled)
                     newlyResolvedProduct->buildData.swap(restoredProduct->buildData);
-                else
-                    productsWithChangedFiles.removeOne(restoredProduct);
                 if (newlyResolvedProduct->buildData) {
                     foreach (BuildGraphNode *node, newlyResolvedProduct->buildData->nodes)
                         node->product = newlyResolvedProduct;
@@ -294,8 +300,8 @@ void BuildGraphLoader::trackProjectChanges()
 
     // Products still left in the list do not exist anymore.
     foreach (const ResolvedProductPtr &removedProduct, allRestoredProducts) {
+        changedProducts.removeOne(removedProduct);
         onProductRemoved(removedProduct, m_result.newlyResolvedProject->buildData.data());
-        productsWithChangedFiles.removeOne(removedProduct);
     }
 
     // Products still left in the list need resolving, either because they are new
@@ -306,15 +312,6 @@ void BuildGraphLoader::trackProjectChanges()
                                                       allNewlyResolvedProducts);
     }
 
-    // For products where only the list of files has changed, we adapt the existing build data
-    // so we won't recompile existing files just because new ones have been added.
-    foreach (const ResolvedProductPtr &product, productsWithChangedFiles) {
-        ResolvedProductPtr freshProduct = freshProductsByName.value(product->uniqueName());
-        if (!freshProduct)
-            continue;
-        onProductFileListChanged(product, freshProduct, oldBuildData.data());
-    }
-
     foreach (const ResolvedProductConstPtr &changedProduct, changedProducts) {
         rescueOldBuildData(changedProduct, freshProductsByName.value(changedProduct->uniqueName()),
                            oldBuildData.data(), childLists,
@@ -379,20 +376,20 @@ bool BuildGraphLoader::hasFileLastModifiedResultChanged(const TopLevelProjectCon
 
 bool BuildGraphLoader::hasProductFileChanged(const QList<ResolvedProductPtr> &restoredProducts,
         const FileTime &referenceTime, QSet<QString> &remainingBuildSystemFiles,
-        QList<ResolvedProductPtr> &productsWithChangedFiles)
+        QList<ResolvedProductPtr> &changedProducts)
 {
     bool hasChanged = false;
     foreach (const ResolvedProductPtr &product, restoredProducts) {
-        const QString fileName = product->location.fileName();
-        const FileInfo pfi(fileName);
-        remainingBuildSystemFiles.remove(fileName);
+        const QString filePath = product->location.filePath();
+        const FileInfo pfi(filePath);
+        remainingBuildSystemFiles.remove(filePath);
         if (!pfi.exists()) {
             m_logger.qbsDebug() << "A product was removed, must re-resolve project";
             hasChanged = true;
         } else if (referenceTime < pfi.lastModified()) {
             m_logger.qbsDebug() << "A product was changed, must re-resolve project";
             hasChanged = true;
-        } else if (!productsWithChangedFiles.contains(product)) {
+        } else if (!changedProducts.contains(product)) {
             foreach (const GroupPtr &group, product->groups) {
                 if (!group->wildcards)
                     continue;
@@ -404,7 +401,7 @@ bool BuildGraphLoader::hasProductFileChanged(const QList<ResolvedProductPtr> &re
                 if (files == wcFiles)
                     continue;
                 hasChanged = true;
-                productsWithChangedFiles += product;
+                changedProducts += product;
                 break;
             }
         }
@@ -428,8 +425,7 @@ bool BuildGraphLoader::hasBuildSystemFileChanged(const QSet<QString> &buildSyste
 
 void BuildGraphLoader::checkAllProductsForChanges(const QList<ResolvedProductPtr> &restoredProducts,
         const QMap<QString, ResolvedProductPtr> &newlyResolvedProductsByName,
-        QList<ResolvedProductPtr> &changedProducts,
-        QList<ResolvedProductPtr> &productsWithChangedFiles)
+        QList<ResolvedProductPtr> &changedProducts)
 {
     foreach (const ResolvedProductPtr &restoredProduct, restoredProducts) {
         if (changedProducts.contains(restoredProduct))
@@ -445,12 +441,12 @@ void BuildGraphLoader::checkAllProductsForChanges(const QList<ResolvedProductPtr
             continue;
         }
 
-        if (!productsWithChangedFiles.contains(restoredProduct)
-                && !sourceArtifactSetsAreEqual(restoredProduct->allFiles(),
-                                                newlyResolvedProduct->allFiles())) {
+        if (!sourceArtifactSetsAreEqual(restoredProduct->allFiles(),
+                                        newlyResolvedProduct->allFiles())) {
             m_logger.qbsDebug() << "File list of product '" << restoredProduct->uniqueName()
                                 << "' was changed.";
-            productsWithChangedFiles += restoredProduct;
+            changedProducts += restoredProduct;
+            continue;
         }
         if (checkProductForChanges(restoredProduct, newlyResolvedProduct)) {
             m_logger.qbsDebug() << "Product '" << restoredProduct->uniqueName()
@@ -497,7 +493,8 @@ bool BuildGraphLoader::checkProductForInstallInfoChanges(const ResolvedProductPt
     // These are not requested from rules at build time, but we still need to take
     // them into account.
     const QStringList specialProperties = QStringList() << QLatin1String("install")
-            << QLatin1String("installDir") << QLatin1String("installPrefix");
+            << QLatin1String("installDir") << QLatin1String("installPrefix")
+            << QLatin1String("installRoot");
     foreach (const QString &key, specialProperties) {
         if (restoredProduct->moduleProperties->qbsPropertyValue(key)
                 != newlyResolvedProduct->moduleProperties->qbsPropertyValue(key)) {
@@ -581,108 +578,6 @@ void BuildGraphLoader::onProductRemoved(const ResolvedProductPtr &product,
     }
 }
 
-void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restoredProduct,
-        const ResolvedProductPtr &newlyResolvedProduct, const ProjectBuildData *oldBuildData)
-{
-    m_logger.qbsDebug() << "[BG] product '" << restoredProduct->uniqueName() << "' changed.";
-
-    QBS_CHECK(newlyResolvedProduct->enabled);
-
-    ArtifactSet artifactsToRemove;
-    QHash<QString, SourceArtifactConstPtr> oldArtifacts, newArtifacts;
-
-    const QList<SourceArtifactPtr> restoredProductAllFiles = restoredProduct->allEnabledFiles();
-    foreach (const SourceArtifactConstPtr &a, restoredProductAllFiles)
-        oldArtifacts.insert(a->absoluteFilePath, a);
-    foreach (const SourceArtifactPtr &a, newlyResolvedProduct->allEnabledFiles()) {
-        newArtifacts.insert(a->absoluteFilePath, a);
-        if (!oldArtifacts.contains(a->absoluteFilePath)) {
-            // artifact added
-            m_logger.qbsDebug() << "[BG] artifact '" << a->absoluteFilePath
-                                << "' added to product " << restoredProduct->uniqueName();
-            Artifact *newArtifact = lookupArtifact(newlyResolvedProduct, oldBuildData,
-                                                   a->absoluteFilePath, true);
-            if (newArtifact) {
-                // User added a source file that was a generated artifact in the previous
-                // build, e.g. a C++ source file that was generated and now is a non-generated
-                // source file.
-                newArtifact->artifactType = Artifact::SourceFile;
-            } else {
-                newArtifact = createArtifact(newlyResolvedProduct, a, m_logger);
-                foreach (FileResourceBase *oldArtifactLookupResult,
-                         oldBuildData->lookupFiles(newArtifact->filePath())) {
-                    if (oldArtifactLookupResult == newArtifact)
-                        continue;
-                    FileDependency *oldFileDependency
-                            = dynamic_cast<FileDependency *>(oldArtifactLookupResult);
-                    if (!oldFileDependency) {
-                        // The source file already exists in another product.
-                        continue;
-                    }
-                    // User added a source file that was recognized as file dependency in the
-                    // previous build, e.g. a C++ header file.
-                    replaceFileDependencyWithArtifact(newlyResolvedProduct,
-                                                      oldFileDependency,
-                                                      newArtifact);
-                }
-            }
-        }
-    }
-
-    foreach (const SourceArtifactPtr &a, restoredProductAllFiles) {
-        const SourceArtifactConstPtr changedArtifact = newArtifacts.value(a->absoluteFilePath);
-        if (!changedArtifact) {
-            // artifact removed
-            m_logger.qbsDebug() << "[BG] artifact '" << a->absoluteFilePath
-                                << "' removed from product " << restoredProduct->uniqueName();
-            Artifact *artifact
-                    = lookupArtifact(restoredProduct, oldBuildData, a->absoluteFilePath, true);
-            QBS_CHECK(artifact);
-            newlyResolvedProduct->topLevelProject()->buildData
-                    ->removeArtifactAndExclusiveDependents(artifact, m_logger, true,
-                                                           &artifactsToRemove);
-            continue;
-        }
-
-        // TODO: overrideFileTags has to be checked for changes as well.
-        if (changedArtifact->fileTags != a->fileTags) {
-            // artifact's filetags have changed
-            m_logger.qbsDebug() << "[BG] filetags have changed for artifact '"
-                    << a->absoluteFilePath << "' from " << a->fileTags << " to "
-                    << changedArtifact->fileTags;
-            Artifact *artifact
-                    = lookupArtifact(restoredProduct, oldBuildData, a->absoluteFilePath, true);
-            QBS_CHECK(artifact);
-            // handle added filetags
-            foreach (const FileTag &addedFileTag, changedArtifact->fileTags - a->fileTags)
-                artifact->addFileTag(addedFileTag);
-
-            // handle removed filetags
-            foreach (const FileTag &removedFileTag, a->fileTags - changedArtifact->fileTags)
-                artifact->removeFileTag(removedFileTag);
-        }
-
-        if (changedArtifact->properties->value() != a->properties->value()) {
-            m_logger.qbsDebug() << "[BG] properties have changed for artifact '"
-                                << a->absoluteFilePath << "'";
-            Artifact * const oldArtifact
-                    = lookupArtifact(restoredProduct, oldBuildData, a->absoluteFilePath, true);
-            QBS_CHECK(oldArtifact);
-            newlyResolvedProduct->topLevelProject()->buildData
-                ->removeArtifactAndExclusiveDependents(oldArtifact, m_logger, true,
-                                                       &artifactsToRemove);
-            createArtifact(newlyResolvedProduct, changedArtifact, m_logger);
-        }
-    }
-
-    // defer destruction of removed artifacts
-    foreach (Artifact *artifact, artifactsToRemove) {
-        if (artifact->artifactType == Artifact::Generated)
-            m_artifactsRemovedFromDisk << artifact->filePath();
-        m_objectsToDelete << artifact;
-    }
-}
-
 static SourceArtifactConstPtr findSourceArtifact(const ResolvedProductConstPtr &product,
         const QString &artifactFilePath, QMap<QString, SourceArtifactConstPtr> &artifactMap)
 {
@@ -733,7 +628,7 @@ bool BuildGraphLoader::checkForPropertyChanges(const TransformerPtr &restoredTra
     }
 
     QMap<QString, SourceArtifactConstPtr> artifactMap;
-    for (QHash<QString, PropertyList>::ConstIterator it =
+    for (QHash<QString, PropertySet>::ConstIterator it =
          restoredTrafo->propertiesRequestedFromArtifactInPrepareScript.constBegin();
          it != restoredTrafo->propertiesRequestedFromArtifactInPrepareScript.constEnd(); ++it) {
         const SourceArtifactConstPtr artifact
@@ -802,10 +697,29 @@ void BuildGraphLoader::replaceFileDependencyWithArtifact(const ResolvedProductPt
     m_objectsToDelete << filedep;
 }
 
+bool BuildGraphLoader::isConfigCompatible()
+{
+    const TopLevelProjectConstPtr restoredProject = m_result.loadedProject;
+    if (m_parameters.finalBuildConfigurationTree() != restoredProject->buildConfiguration())
+        return false;
+    for (QVariantMap::ConstIterator it = restoredProject->profileConfigs.constBegin();
+         it != restoredProject->profileConfigs.constEnd(); ++it) {
+        const QVariantMap buildConfig = SetupProjectParameters::expandedBuildConfiguration(
+                    m_parameters.settingsDirectory(), it.key(), m_parameters.buildVariant());
+        const QVariantMap newConfig = SetupProjectParameters::finalBuildConfigurationTree(
+                    buildConfig, m_parameters.overriddenValues(), m_parameters.buildRoot(),
+                    m_parameters.topLevelProfile());
+        if (newConfig != it.value())
+            return false;
+    }
+    return true;
+}
+
 void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restoredProduct,
         const ResolvedProductPtr &newlyResolvedProduct, ProjectBuildData *oldBuildData,
         const ChildListHash &childLists, const AllRescuableArtifactData &existingRad)
 {
+    QBS_CHECK(newlyResolvedProduct);
     if (!restoredProduct->enabled || !newlyResolvedProduct->enabled)
         return;
 
