@@ -1,7 +1,7 @@
 #############################################################################
 ##
-## Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-## Contact: http://www.qt-project.org/legal
+## Copyright (C) 2015 The Qt Company Ltd.
+## Contact: http://www.qt.io/licensing
 ##
 ## This file is part of Qt Creator.
 ##
@@ -9,8 +9,8 @@
 ## Licensees holding valid commercial Qt licenses may use this file in
 ## accordance with the commercial license agreement provided with the
 ## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and Digia.  For licensing terms and
-## conditions see http://www.qt.io/licensing.  For further information
+## a written agreement between you and The Qt Company.  For licensing terms and
+## conditions see http://www.qt.io/terms-conditions.  For further information
 ## use the contact form at http://www.qt.io/contact-us.
 ##
 ## GNU Lesser General Public License Usage
@@ -20,44 +20,47 @@
 ## LICENSE.LGPLv3 included in the packaging of this file.  Please review the
 ## following information to ensure the GNU Lesser General Public License
 ## requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-# http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+## http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 ##
-## In addition, as a special exception, Digia gives you certain additional
-## rights.  These rights are described in the Digia Qt LGPL Exception
+## In addition, as a special exception, The Qt Company gives you certain additional
+## rights.  These rights are described in The Qt Company LGPL Exception
 ## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 ##
 #############################################################################
 
 import operator
 
-isQt4Build = False
-
 # for easier re-usage (because Python hasn't an enum type)
 class Targets:
-    DESKTOP_474_GCC = 1
-    DESKTOP_480_GCC = 2
-    SIMULATOR = 4
-    MAEMO5 = 8
-    HARMATTAN = 16
-    EMBEDDED_LINUX = 32
-    DESKTOP_480_MSVC2010 = 64
-    DESKTOP_521_DEFAULT = 128
-    DESKTOP_531_DEFAULT = 256
+    ALL_TARGETS = map(lambda x: 2 ** x , range(9))
+
+    (DESKTOP_474_GCC,
+     DESKTOP_480_DEFAULT,
+     SIMULATOR,
+     MAEMO5,
+     HARMATTAN,
+     EMBEDDED_LINUX,
+     DESKTOP_521_DEFAULT,
+     DESKTOP_531_DEFAULT,
+     DESKTOP_541_GCC) = ALL_TARGETS
 
     @staticmethod
     def desktopTargetClasses():
-        desktopTargets = (Targets.DESKTOP_474_GCC | Targets.DESKTOP_480_GCC
-                          | Targets.DESKTOP_521_DEFAULT | Targets.DESKTOP_531_DEFAULT)
-        if platform.system() in ('Windows', 'Microsoft'):
-            desktopTargets |= Targets.DESKTOP_480_MSVC2010
+        desktopTargets = (sum(Targets.ALL_TARGETS) & ~Targets.SIMULATOR & ~Targets.MAEMO5
+                          & ~Targets.HARMATTAN & ~Targets.EMBEDDED_LINUX)
+        if platform.system() == 'Darwin':
+            desktopTargets &= ~Targets.DESKTOP_541_GCC
         return desktopTargets
 
     @staticmethod
     def getStringForTarget(target):
         if target == Targets.DESKTOP_474_GCC:
             return "Desktop 474 GCC"
-        if target == Targets.DESKTOP_480_GCC:
-            return "Desktop 480 GCC"
+        elif target == Targets.DESKTOP_480_DEFAULT:
+            if platform.system() in ('Windows', 'Microsoft'):
+                return "Desktop 480 MSVC2010"
+            else:
+                return "Desktop 480 GCC"
         elif target == Targets.MAEMO5:
             return "Fremantle"
         elif target == Targets.SIMULATOR:
@@ -66,12 +69,12 @@ class Targets:
             return "Harmattan"
         elif target == Targets.EMBEDDED_LINUX:
             return "Embedded Linux"
-        elif target == Targets.DESKTOP_480_MSVC2010:
-            return "Desktop 480 MSVC2010"
         elif target == Targets.DESKTOP_521_DEFAULT:
             return "Desktop 521 default"
         elif target == Targets.DESKTOP_531_DEFAULT:
             return "Desktop 531 default"
+        elif target == Targets.DESKTOP_541_GCC:
+            return "Desktop 541 GCC"
         else:
             return None
 
@@ -87,10 +90,7 @@ class Targets:
 
     @staticmethod
     def intToArray(targets):
-        available = [Targets.DESKTOP_474_GCC, Targets.DESKTOP_480_GCC, Targets.SIMULATOR, Targets.MAEMO5,
-                     Targets.HARMATTAN, Targets.EMBEDDED_LINUX, Targets.DESKTOP_480_MSVC2010,
-                     Targets.DESKTOP_521_DEFAULT, Targets.DESKTOP_531_DEFAULT]
-        return filter(lambda x: x & targets == x, available)
+        return filter(lambda x: x & targets, Targets.ALL_TARGETS)
 
     @staticmethod
     def arrayToInt(targetArr):
@@ -107,10 +107,7 @@ class ProjectSettings:
 
 # this class defines some constants for the views of the creator's MainWindow
 class ViewConstants:
-    if isQt4Build:
-        EDIT, DESIGN, DEBUG, PROJECTS, ANALYZE, HELP = range(6)
-    else:
-        WELCOME, EDIT, DESIGN, DEBUG, PROJECTS, ANALYZE, HELP = range(7)
+    WELCOME, EDIT, DESIGN, DEBUG, PROJECTS, ANALYZE, HELP = range(7)
     FIRST_AVAILABLE = 0
     # always adjust the following to the highest value of the available ViewConstants when adding new
     LAST_AVAILABLE = HELP
@@ -120,7 +117,7 @@ class ViewConstants:
     # if the provided argument does not match any of the ViewConstants it returns None
     @staticmethod
     def getToolTipForViewTab(viewTab):
-        if not isQt4Build and viewTab == ViewConstants.WELCOME:
+        if viewTab == ViewConstants.WELCOME:
             toolTip = ur'Switch to <b>Welcome</b> mode <span style="color: gray; font-size: small">(Ctrl\+|\u2303)%d</span>'
         elif viewTab == ViewConstants.EDIT:
             toolTip = ur'Switch to <b>Edit</b> mode <span style="color: gray; font-size: small">(Ctrl\+|\u2303)%d</span>'
